Package: climniche
Type: Package
Title: Niche Climate Exposure
Version: 0.0.1
Authors@R: c(
    person("Bohao", "He", role = c("aut", "cre"), email = "bohao.he@polimi.it")
    )
Description: Assesses niche climate exposure by interpreting projected climate
    change relative to the climate conditions a species currently occupies.
    Using occurrence records, range data or thresholded SDM suitability maps,
    current environmental rasters and future projections, the package separates
    climate change amount, change in distance to the current niche centre,
    composition change and exceedance beyond an empirical niche boundary.
License: GPL (>= 3)
URL: https://github.com/Bohao0813/climniche,
        https://bohao0813.github.io/climniche/
BugReports: https://github.com/Bohao0813/climniche/issues
Encoding: UTF-8
Imports: grid, methods
Suggests: ggplot2, knitr, patchwork, raster, rmarkdown, terra
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-26 13:11:44 UTC; Bohao He
Author: Bohao He [aut, cre]
Maintainer: Bohao He <bohao.he@polimi.it>
Repository: CRAN
Date/Publication: 2026-05-29 12:20:02 UTC
Built: R 4.5.2; ; 2026-05-29 13:30:10 UTC; unix

Package: climniche
Type: Package
Title: Niche Climate Exposure
Version: 0.0.1
Authors@R: c(
    person("Bohao", "He", role = c("aut", "cre"), email = "bohao.he@polimi.it")
    )
Description: Assesses niche climate exposure by interpreting projected climate
    change relative to the climate conditions a species currently occupies.
    Using occurrence records, range data or thresholded SDM suitability maps,
    current environmental rasters and future projections, the package separates
    climate change amount, change in distance to the current niche centre,
    composition change and exceedance beyond an empirical niche boundary.
License: GPL (>= 3)
URL: https://github.com/Bohao0813/climniche,
        https://bohao0813.github.io/climniche/
BugReports: https://github.com/Bohao0813/climniche/issues
Encoding: UTF-8
Imports: grid, methods
Suggests: ggplot2, knitr, patchwork, raster, rmarkdown, terra
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-26 13:11:44 UTC; Bohao He
Author: Bohao He [aut, cre]
Maintainer: Bohao He <bohao.he@polimi.it>
Repository: CRAN
Date/Publication: 2026-05-29 12:20:02 UTC
Built: R 4.6.0; ; 2026-05-29 14:12:58 UTC; unix

export(fit_climniche)
export(fit_climniche_raster)
export(fit_climniche_terra)
export(climniche_diagram_data)
export(climniche_showcase_data)
export(niche_metric)
export(niche_potential)
export(niche_radius)
export(plot_climniche_map)
export(plot_climniche_classes)
export(plot_climniche_exposure)
export(plot_climniche_class_summary)
export(plot_climniche_distribution)
export(plot_climniche_report)
export(plot_climniche_diagram)
export(plot_climniche_showcase)
export(plot_climniche_variable_contribution)
export(plot_variable_contribution)
export(climniche_table)
export(climniche_summary)
export(climniche_report)
export(write_climniche_report)
export(boundary_exceedance)
export(niche_percentile)
export(variable_contribution)
export(simulate_climniche)
importFrom(methods,is)
importFrom(methods,slot)
importFrom(methods,slotNames)
S3method(summary,climniche_fit)
S3method(plot,climniche_fit)
S3method(print,climniche_report)

<p align="center" style="margin-top: 28px; margin-bottom: 24px;">
  <img src="man/figures/climniche-hex.svg" width="190" alt="climniche logo" />
</p>

# climniche

[![R-CMD-check](https://github.com/Bohao0813/climniche/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Bohao0813/climniche/actions/workflows/R-CMD-check.yaml)
[![pkgdown](https://github.com/Bohao0813/climniche/actions/workflows/pkgdown.yaml/badge.svg)](https://github.com/Bohao0813/climniche/actions/workflows/pkgdown.yaml)

Website: <https://bohao0813.github.io/climniche/>

Pages: [Examples](https://bohao0813.github.io/climniche/articles/climniche-examples.html) |
[Reference](https://bohao0813.github.io/climniche/reference/index.html) |
[News](https://bohao0813.github.io/climniche/news/index.html)

![Niche climate exposure concept](man/figures/niche-climate-exposure.svg)

`climniche` quantifies niche climate exposure: the amount and direction of
projected climate change at cells currently associated with a taxon, measured
relative to its realised climatic niche.

`climniche` separates four quantities in exposure maps:

- `climate_change_amount`: how far climate moves between the current and future
  period;
- `niche_distance_change`: whether future climate becomes closer to or farther
  from the realised niche centre;
- `composition_change`: change in climate composition that does not mainly
  alter distance from the niche centre;
- `outside_niche_exceedance`: how far future climate lies beyond an empirical
  boundary of current niche conditions.

Together, these quantities distinguish local environmental change from change
that moves current occurrence, range, or SDM cells outside the realised niche.

## Inputs and outputs

`climniche` accepts extracted environmental matrices and aligned `raster` or
`terra` layers. The current reference cells can be supplied as occurrences,
a range raster, or a continuous SDM suitability raster with a chosen threshold.

The outputs are cell-level tables, maps, exposure classes, variable
contributions, report text, and figure data.

## Basic use

```r
library(climniche)

sim <- simulate_climniche()

fit <- fit_climniche(
  current = sim$current,
  future = sim$future_away,
  occupied = sim$occupied,
  center = sim$center,
  sensitivity = sim$sensitivity
)

climniche_summary(fit)
climniche_report(fit, species = "example species")
plot_climniche_diagram(fit)
```

For spatial data, use `fit_climniche_raster()` or `fit_climniche_terra()`.
Both functions accept binary and continuous occupied rasters. `domain` can be
used to restrict the calculation to a study area such as a marine region or a
modelled accessible area.

library(climniche)

sim <- simulate_climniche(n = 1000, p = 3, rho = 0.5, shift = 0.5)

fit <- fit_climniche(
  current = sim$current,
  future = sim$future_away,
  occupied = sim$occupied,
  center = sim$center,
  sensitivity = sim$sensitivity,
  scale = FALSE
)

report <- climniche_report(fit, species = "simulated species")
print(report)

if (requireNamespace("ggplot2", quietly = TRUE)) {
  plot_climniche_exposure(fit)
  plot_climniche_class_summary(fit)
  plot_climniche_variable_contribution(fit)
}

# write_climniche_report(report, "climniche_report.md")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", fig.align = "center")

## -----------------------------------------------------------------------------
library(climniche)

case_path <- system.file("extdata/mediterranean_anchovy", package = "climniche")
summary_tab <- read.csv(file.path(case_path, "anchovy_climniche_summary.csv"))
class_tab <- read.csv(file.path(case_path, "anchovy_climniche_classes.csv"))
variable_tab <- read.csv(file.path(case_path, "anchovy_climniche_top_variables.csv"))
records <- read.csv(file.path(case_path, "anchovy_clean_obis_records.csv"))

## -----------------------------------------------------------------------------
summary_tab[, c(
  "n", "boundary_quantile", "mean_climate_change_amount",
  "mean_niche_distance_change", "prop_niche_divergence",
  "prop_niche_exceedance", "prop_niche_convergence"
)]

head(class_tab)
head(variable_tab)
nrow(records)

## ----echo = FALSE, out.width = "100%"-----------------------------------------
knitr::include_graphics("figures/anchovy-mediterranean-maps.png")

## ----echo = FALSE, out.width = "80%"------------------------------------------
knitr::include_graphics("figures/anchovy-climniche-diagram.png")

## ----echo = FALSE, out.width = "95%"------------------------------------------
knitr::include_graphics("figures/anchovy-climniche-showcase.png")

## -----------------------------------------------------------------------------
class_tab[, c("class", "proportion")]
variable_tab[, c("variable", "mean_contribution", "interpretation")]

#' Classify projected climate change relative to the current niche
#'
#' @param climate_change_amount Numeric climate change amount.
#' @param niche_distance_change Numeric change in distance to the current niche.
#' @param outside_niche_exceedance Numeric exceedance beyond the niche boundary.
#' @param composition_change Optional composition change.
#' @param contribution Optional variable contribution matrix.
#' @param tolerance Optional niche distance change tolerance. If NULL, the 10th
#'   percentile of absolute niche distance change is used.
#' @param stable_quantile Quantile of climate change amount used to mark weak
#'   climate change.
#' @param conflict_ratio Retained for compatibility with the fitting workflow.
#'
#' @return Factor of climniche classes.
#' @noRd
classify_exposure <- function(climate_change_amount, niche_distance_change,
                              outside_niche_exceedance,
                              composition_change = NULL,
                              contribution = NULL,
                              tolerance = NULL, stable_quantile = 0.25,
                              conflict_ratio = NULL) {
  if (is.null(tolerance)) {
    tolerance <- as.numeric(stats::quantile(abs(niche_distance_change),
                                            probs = 0.10, names = FALSE,
                                            na.rm = TRUE, type = 8))
  }
  stable_cut <- as.numeric(stats::quantile(
    climate_change_amount,
    probs = stable_quantile,
    names = FALSE,
    na.rm = TRUE,
    type = 8
  ))
  cls <- rep("changed composition, similar distance",
             length(climate_change_amount))
  cls[niche_distance_change < -tolerance] <- "closer to current niche"
  cls[niche_distance_change > tolerance] <- "farther from current niche"
  cls[outside_niche_exceedance > 0] <- "outside current niche boundary"
  cls[climate_change_amount <= stable_cut & outside_niche_exceedance == 0 &
        abs(niche_distance_change) <= tolerance] <- "little climate niche change"

  if (!is.null(composition_change)) {
    low_composition <- composition_change <= stats::quantile(
      composition_change,
      probs = stable_quantile,
      names = FALSE,
      na.rm = TRUE,
      type = 8
    )
    cls[low_composition & climate_change_amount <= stable_cut &
          outside_niche_exceedance == 0 &
          abs(niche_distance_change) <= tolerance] <- "little climate niche change"
  }

  factor(cls, levels = c(
    "little climate niche change",
    "closer to current niche",
    "farther from current niche",
    "outside current niche boundary",
    "changed composition, similar distance"
  ))
}

mixed_variable_response <- function(contribution, outside_niche_exceedance = NULL,
                                    conflict_ratio = 0.25) {
  if (is.null(contribution) || is.null(conflict_ratio)) {
    n <- if (is.null(outside_niche_exceedance)) 0 else length(outside_niche_exceedance)
    return(rep(FALSE, n))
  }
  pos <- rowSums(pmax(contribution, 0))
  neg <- rowSums(abs(pmin(contribution, 0)))
  total <- pos + neg
  mixed <- total > 0 & pmin(pos, neg) / total >= conflict_ratio
  if (!is.null(outside_niche_exceedance)) {
    mixed <- mixed & outside_niche_exceedance == 0
  }
  mixed
}

if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    "class",
    "class_id",
    "cell_weight",
    "climate_change_amount",
    "contribution",
    "current",
    "current_axis1",
    "current_axis2",
    "boundary_quantile",
    "future",
    "future_axis1",
    "future_axis2",
    "label",
    "metric",
    "mean_contribution",
    "niche_distance_change",
    "prop",
    "prop_exceeded",
    "proportion",
    "value",
    "variable",
    "x",
    "axis1",
    "axis2",
    "x_mid",
    "x_value",
    "y",
    "y_mid",
    "y_value"
  ))
}

#' Fit climniche to Raster* climate layers
#'
#' @param current Raster* object of current climate layers.
#' @param future Raster* object of future climate layers with the same geometry
#'   and number of layers as `current`.
#' @param occupied Optional RasterLayer indicating current occurrence or range
#'   cells. Cells with values greater than `occupied_threshold` are treated as
#'   current occurrence cells, so binary range maps and continuous suitability
#'   rasters are both supported.
#' @param occupied_threshold Numeric threshold used when `occupied` is a raster.
#' @param domain Optional RasterLayer limiting the cells to analyse. Values
#'   greater than `domain_threshold` define the analysis domain.
#' @param domain_threshold Numeric threshold used when `domain` is supplied.
#' @param ... Arguments passed to `fit_climniche()`.
#'
#' @return A `climniche_fit` object with an additional `rasters` element containing
#'   RasterLayer outputs.
#' @noRd
.fit_climniche_raster <- function(current, future, occupied = NULL,
                                  occupied_threshold = 0, domain = NULL,
                                  domain_threshold = 0, ...) {
  if (!requireNamespace("raster", quietly = TRUE)) {
    stop("The raster package is required for fit_climniche_raster().",
         call. = FALSE)
  }
  if (!methods::is(current, "Raster") || !methods::is(future, "Raster")) {
    stop("current and future must be raster Raster* objects.", call. = FALSE)
  }
  if (!raster::compareRaster(current, future, stopiffalse = FALSE)) {
    stop("current and future rasters must have matching geometry.", call. = FALSE)
  }

  x0 <- raster::getValues(current)
  x1 <- raster::getValues(future)
  complete <- stats::complete.cases(x0) & stats::complete.cases(x1)
  if (!is.null(domain)) {
    if (!methods::is(domain, "Raster")) {
      stop("domain must be NULL or a RasterLayer.", call. = FALSE)
    }
    if (!raster::compareRaster(raster::raster(current), domain,
                               stopiffalse = FALSE)) {
      stop("domain raster must match current raster geometry.",
           call. = FALSE)
    }
    dom <- raster::getValues(domain)
    complete <- complete & !is.na(dom) & dom > domain_threshold
  }
  if (!any(complete)) {
    stop("No analysable cells remain after applying environmental and domain masks.",
         call. = FALSE)
  }

  if (is.null(occupied)) {
    occupied_idx <- seq_len(sum(complete))
  } else {
    if (!methods::is(occupied, "Raster")) {
      stop("occupied must be NULL or a RasterLayer.", call. = FALSE)
    }
    if (!raster::compareRaster(raster::raster(current), occupied,
                               stopiffalse = FALSE)) {
      stop("occupied raster must match current raster geometry.", call. = FALSE)
    }
    occ <- raster::getValues(occupied)
    occupied_idx <- which((!is.na(occ) & occ > occupied_threshold)[complete])
  }
  if (!length(occupied_idx)) {
    stop("No occupied cells found after applying occupied_threshold.",
         call. = FALSE)
  }

  fit <- .fit_climniche_matrix(
    current = x0[complete, , drop = FALSE],
    future = x1[complete, , drop = FALSE],
    occupied = occupied_idx,
    ...
  )

  fit$rasters <- list(
    climate_change_amount = .values_to_raster(current, fit$climate_change_amount,
                                              complete),
    niche_distance_change = .values_to_raster(current, fit$niche_distance_change,
                                             complete),
    composition_change = .values_to_raster(
      current,
      fit$composition_change,
      complete
    ),
    change_alignment = .values_to_raster(
      current,
      fit$change_alignment,
      complete
    ),
    outside_niche_exceedance = .values_to_raster(current, fit$outside_niche_exceedance,
                                                 complete),
    classification = .values_to_raster(
      current,
      as.integer(fit$classification),
      complete
    )
  )
  fit$class_lookup <- data.frame(
    id = seq_along(levels(fit$classification)),
    class = levels(fit$classification)
  )
  fit$raster_complete <- complete
  fit
}

.values_to_raster <- function(template, values_complete, complete) {
  r <- raster::raster(template)
  out <- rep(NA_real_, raster::ncell(r))
  out[complete] <- values_complete
  raster::values(r) <- out
  r
}

#' Fit climniche to terra SpatRaster environmental layers
#'
#' @param current SpatRaster of current environmental layers.
#' @param future SpatRaster of future environmental layers with the same
#'   geometry and number of layers as `current`.
#' @param occupied Optional SpatRaster layer indicating current occurrence,
#'   range, suitability or probability cells. Values greater than
#'   `occupied_threshold` are treated as current occurrence cells.
#' @param occupied_threshold Numeric threshold used when `occupied` is supplied.
#' @param domain Optional one-layer SpatRaster limiting the cells to analyse.
#'   Values greater than `domain_threshold` define the analysis domain.
#' @param domain_threshold Numeric threshold used when `domain` is supplied.
#' @param ... Arguments passed to `fit_climniche()`.
#'
#' @return A `climniche_fit` object with an additional `rasters` element containing
#'   SpatRaster outputs.
#' @noRd
.fit_climniche_terra <- function(current, future, occupied = NULL,
                                 occupied_threshold = 0, domain = NULL,
                                 domain_threshold = 0, ...) {
  if (!requireNamespace("terra", quietly = TRUE)) {
    stop("The terra package is required for fit_climniche_terra().",
         call. = FALSE)
  }
  if (!methods::is(current, "SpatRaster") ||
      !methods::is(future, "SpatRaster")) {
    stop("current and future must be terra SpatRaster objects.",
         call. = FALSE)
  }
  if (!terra::compareGeom(current, future, stopOnError = FALSE)) {
    stop("current and future rasters must have matching geometry.",
         call. = FALSE)
  }

  x0 <- terra::values(current)
  x1 <- terra::values(future)
  complete <- stats::complete.cases(x0) & stats::complete.cases(x1)
  if (!is.null(domain)) {
    if (!methods::is(domain, "SpatRaster")) {
      stop("domain must be NULL or a terra SpatRaster.", call. = FALSE)
    }
    if (terra::nlyr(domain) != 1) {
      stop("domain must have one layer.", call. = FALSE)
    }
    if (!terra::compareGeom(current[[1]], domain, stopOnError = FALSE)) {
      stop("domain raster must match current raster geometry.",
           call. = FALSE)
    }
    dom <- terra::values(domain)[, 1]
    complete <- complete & !is.na(dom) & dom > domain_threshold
  }
  if (!any(complete)) {
    stop("No analysable cells remain after applying environmental and domain masks.",
         call. = FALSE)
  }

  if (is.null(occupied)) {
    occupied_idx <- seq_len(sum(complete))
  } else {
    if (!methods::is(occupied, "SpatRaster")) {
      stop("occupied must be NULL or a terra SpatRaster.", call. = FALSE)
    }
    if (terra::nlyr(occupied) != 1) {
      stop("occupied must have one layer.", call. = FALSE)
    }
    if (!terra::compareGeom(current[[1]], occupied, stopOnError = FALSE)) {
      stop("occupied raster must match current raster geometry.",
           call. = FALSE)
    }
    occ <- terra::values(occupied)[, 1]
    occupied_idx <- which((!is.na(occ) & occ > occupied_threshold)[complete])
  }
  if (!length(occupied_idx)) {
    stop("No occupied cells found after applying occupied_threshold.",
         call. = FALSE)
  }

  fit <- .fit_climniche_matrix(
    current = x0[complete, , drop = FALSE],
    future = x1[complete, , drop = FALSE],
    occupied = occupied_idx,
    ...
  )

  fit$rasters <- list(
    climate_change_amount = .values_to_spatraster(
      current, fit$climate_change_amount, complete
    ),
    niche_distance_change = .values_to_spatraster(
      current, fit$niche_distance_change, complete
    ),
    composition_change = .values_to_spatraster(
      current, fit$composition_change, complete
    ),
    change_alignment = .values_to_spatraster(
      current, fit$change_alignment, complete
    ),
    outside_niche_exceedance = .values_to_spatraster(
      current, fit$outside_niche_exceedance, complete
    ),
    classification = .values_to_spatraster(
      current, as.integer(fit$classification), complete
    )
  )
  fit$class_lookup <- data.frame(
    id = seq_along(levels(fit$classification)),
    class = levels(fit$classification)
  )
  fit$raster_complete <- complete
  fit
}

.values_to_spatraster <- function(template, values_complete, complete) {
  r <- template[[1]]
  out <- rep(NA_real_, terra::ncell(r))
  out[complete] <- values_complete
  terra::values(r) <- out
  r
}

.showcase_breaks <- function(x, bins) {
  x <- x[is.finite(x)]
  if (!length(x)) {
    return(seq(0, 1, length.out = bins + 1L))
  }
  rng <- range(x, na.rm = TRUE)
  if (!is.finite(diff(rng)) || diff(rng) <= 0) {
    pad <- if (rng[1] == 0) 0.5 else abs(rng[1]) * 0.05
    rng <- rng + c(-pad, pad)
  } else {
    pad <- diff(rng) * 0.001
    rng <- rng + c(-pad, pad)
  }
  seq(rng[1], rng[2], length.out = bins + 1L)
}

.dominant_exposure_bins <- function(tab, bins = 45L) {
  bins <- max(8L, as.integer(bins)[1])
  x_breaks <- .showcase_breaks(tab$climate_change_amount, bins)
  y_breaks <- .showcase_breaks(tab$niche_distance_change, bins)
  x_id <- findInterval(tab$climate_change_amount, x_breaks,
                       all.inside = TRUE)
  y_id <- findInterval(tab$niche_distance_change, y_breaks,
                       all.inside = TRUE)
  ok <- is.finite(tab$climate_change_amount) &
    is.finite(tab$niche_distance_change) &
    !is.na(tab$class)
  dat <- data.frame(
    x_id = x_id[ok],
    y_id = y_id[ok],
    class = as.character(tab$class[ok]),
    stringsAsFactors = FALSE
  )
  if (!nrow(dat)) {
    return(data.frame(
      x_mid = numeric(), y_mid = numeric(), class = factor(),
      class_count = integer(), total_count = integer(),
      proportion_in_bin = numeric(), cell_weight = numeric(),
      stringsAsFactors = FALSE
    ))
  }

  counts <- stats::aggregate(
    list(class_count = rep(1L, nrow(dat))),
    by = list(x_id = dat$x_id, y_id = dat$y_id, class = dat$class),
    FUN = sum
  )
  counts <- counts[order(counts$x_id, counts$y_id, -counts$class_count,
                         counts$class), , drop = FALSE]
  bin_key <- paste(counts$x_id, counts$y_id, sep = ":")
  total <- stats::aggregate(
    list(total_count = counts$class_count),
    by = list(x_id = counts$x_id, y_id = counts$y_id),
    FUN = sum
  )
  total_key <- paste(total$x_id, total$y_id, sep = ":")
  out <- counts[!duplicated(bin_key), , drop = FALSE]
  out_key <- paste(out$x_id, out$y_id, sep = ":")
  out$total_count <- total$total_count[match(out_key, total_key)]
  out$proportion_in_bin <- out$class_count / out$total_count
  out$x_mid <- (x_breaks[out$x_id] + x_breaks[out$x_id + 1L]) / 2
  out$y_mid <- (y_breaks[out$y_id] + y_breaks[out$y_id + 1L]) / 2
  out$x_width <- diff(x_breaks)[out$x_id]
  out$y_height <- diff(y_breaks)[out$y_id]
  out$cell_weight <- sqrt(out$total_count / max(out$total_count, na.rm = TRUE))
  out$class <- factor(out$class, levels = names(.class_colours()))
  out[, c("x_mid", "y_mid", "class", "class_count", "total_count",
          "proportion_in_bin", "cell_weight", "x_width", "y_height")]
}

.showcase_metric_distribution <- function(tab) {
  metrics <- c(
    climate_change_amount = "Climate change\namount",
    niche_distance_change = "Niche distance\nchange",
    composition_change = "Composition\nchange",
    outside_niche_exceedance = "Niche boundary\nexceedance"
  )
  out <- do.call(rbind, lapply(names(metrics), function(nm) {
    data.frame(
      metric = unname(metrics[nm]),
      value = tab[[nm]],
      stringsAsFactors = FALSE
    )
  }))
  out <- out[is.finite(out$value), , drop = FALSE]
  out$metric <- factor(out$metric, levels = unname(metrics))
  out
}

#' Build data for the climniche showcase figure
#'
#' @param x A fitted climniche object.
#' @param scope `"current"` for current occurrence, range or thresholded SDM
#'   cells; `"all"` for all evaluated cells.
#' @param max_points Maximum number of cells to keep for the exposure plane.
#' @param seed Random seed used when subsampling cells.
#' @param plane_bins Number of fixed bins used to summarize the exposure plane.
#' @param boundary_probs Boundary quantiles used for the sensitivity curve.
#' @param top_variables Number of variables to show.
#'
#' @return A list of data frames used by `plot_climniche_showcase()`.
#' @export
climniche_showcase_data <- function(x, scope = c("current", "all"),
                                    max_points = 6000L, seed = 1L,
                                    plane_bins = 45L,
                                    boundary_probs = seq(0.50, 0.99, 0.01),
                                    top_variables = 6L) {
  if (!inherits(x, "climniche_fit")) {
    stop("x must be a fitted climniche object.", call. = FALSE)
  }
  scope <- match.arg(scope)
  tab <- climniche_table(x, scope = scope)
  tab$class <- factor(as.character(tab$class), levels = names(.class_colours()))

  plane <- tab
  if (nrow(plane) > max_points) {
    set.seed(seed)
    plane <- plane[sample(seq_len(nrow(plane)), max_points), , drop = FALSE]
  }

  class_counts <- as.data.frame(table(tab$class), stringsAsFactors = FALSE)
  names(class_counts) <- c("class", "count")
  class_counts$proportion <- class_counts$count / nrow(tab)
  class_counts <- class_counts[class_counts$count > 0, , drop = FALSE]
  class_counts$class <- factor(class_counts$class,
                               levels = rev(as.character(class_counts$class)))
  class_counts$label <- ifelse(
    class_counts$proportion > 0 & class_counts$proportion < 0.01,
    "<1%",
    paste0(round(100 * class_counts$proportion), "%")
  )

  idx <- if (scope == "current") x$occupied else seq_len(nrow(x$current))
  vals <- colMeans(x$variable_contribution[idx, , drop = FALSE],
                   na.rm = TRUE)
  variables <- data.frame(
    variable = names(vals),
    mean_contribution = as.numeric(vals),
    abs_mean_contribution = abs(as.numeric(vals)),
    direction = ifelse(vals >= 0,
                       "less similar to current niche",
                       "more similar to current niche"),
    stringsAsFactors = FALSE
  )
  variables <- variables[order(variables$abs_mean_contribution,
                               decreasing = TRUE), , drop = FALSE]
  variables <- utils::head(variables, top_variables)
  variables$variable <- factor(variables$variable,
                               levels = rev(variables$variable))

  boundary_probs <- sort(unique(boundary_probs))
  boundary_probs <- boundary_probs[is.finite(boundary_probs) &
                                     boundary_probs > 0 & boundary_probs < 1]
  boundary <- do.call(rbind, lapply(boundary_probs, function(q) {
    b_potential <- as.numeric(stats::quantile(
      x$psi_current[x$occupied],
      probs = q,
      names = FALSE,
      na.rm = TRUE,
      type = 8
    ))
    b_radius <- sqrt(pmax(0, b_potential))
    exceed <- pmax(0, x$niche_radius_future[idx] - b_radius)
    data.frame(
      boundary_quantile = q,
      boundary_distance = b_radius,
      prop_exceeded = mean(exceed > 0, na.rm = TRUE),
      mean_exceedance = mean(exceed, na.rm = TRUE)
    )
  }))

  out <- list(
    plane = plane,
    plane_bins = .dominant_exposure_bins(tab, bins = plane_bins),
    classes = class_counts,
    variables = variables,
    boundary = boundary,
    metrics = .showcase_metric_distribution(tab),
    settings = data.frame(
      scope = scope,
      n_cells = nrow(tab),
      sampled_cells = nrow(plane),
      plane_bins = plane_bins,
      fitted_boundary_quantile = x$boundary_quantile,
      fitted_boundary_distance = x$boundary_radius
    )
  )
  class(out) <- "climniche_showcase_data"
  out
}

#' Plot the climniche showcase figure
#'
#' @param x A fitted climniche object or data returned by
#'   `climniche_showcase_data()`.
#' @param scope `"current"` for current occurrence, range or thresholded SDM
#'   cells; `"all"` for all evaluated cells.
#' @param max_points Maximum number of cells to draw in the exposure plane.
#' @param seed Random seed used when subsampling cells.
#' @param plane_bins Number of fixed bins used to summarize the exposure plane.
#' @param boundary_probs Boundary quantiles used for the sensitivity curve.
#' @param top_variables Number of variables to show.
#' @param variable_labels Optional named vector replacing variable labels.
#' @param title Optional overall title when `patchwork` is installed.
#'
#' @return A patchwork object when `patchwork` is installed, otherwise a named
#'   list of ggplot objects.
#' @export
plot_climniche_showcase <- function(x, scope = c("current", "all"),
                                    max_points = 6000L, seed = 1L,
                                    plane_bins = 45L,
                                    boundary_probs = seq(0.50, 0.99, 0.01),
                                    top_variables = 6L,
                                    variable_labels = NULL,
                                    title = NULL) {
  .need_ggplot2()
  if (!inherits(x, "climniche_showcase_data")) {
    x <- climniche_showcase_data(
      x,
      scope = scope,
      max_points = max_points,
      seed = seed,
      plane_bins = plane_bins,
      boundary_probs = boundary_probs,
      top_variables = top_variables
    )
  }
  if (!is.null(variable_labels)) {
    idx <- match(as.character(x$variables$variable), names(variable_labels))
    replace <- !is.na(idx)
    levels(x$variables$variable)[match(
      as.character(x$variables$variable)[replace],
      levels(x$variables$variable)
    )] <- unname(variable_labels[idx[replace]])
  }

  class_cols <- .class_colours()
  class_labs <- c(
    "little climate niche change" = "little climate\nniche change",
    "closer to current niche" = "closer to\ncurrent niche",
    "farther from current niche" = "farther from\ncurrent niche",
    "outside current niche boundary" = "outside current\nniche boundary",
    "changed composition, similar distance" = "changed composition,\nsimilar distance"
  )
  tile_width <- if (nrow(x$plane_bins)) x$plane_bins$x_width[1] else 1
  tile_height <- if (nrow(x$plane_bins)) x$plane_bins$y_height[1] else 1
  p_plane <- ggplot2::ggplot(
    x$plane_bins,
    ggplot2::aes(x = x_mid, y = y_mid, fill = class,
                 alpha = cell_weight)
  ) +
    ggplot2::geom_hline(yintercept = 0, linewidth = 0.25,
                        linetype = 2, colour = "grey45") +
    ggplot2::geom_tile(width = tile_width, height = tile_height,
                       colour = NA) +
    ggplot2::scale_fill_manual(
      values = class_cols,
      labels = class_labs,
      drop = TRUE,
      guide = ggplot2::guide_legend(nrow = 2, byrow = TRUE)
    ) +
    ggplot2::scale_alpha(range = c(0.25, 0.95), guide = "none") +
    ggplot2::labs(
      title = "(a) Binned exposure plane",
      subtitle = "Squares are fixed bins; colours follow panel (b)",
      x = "Climate change amount",
      y = "Niche distance change",
      fill = "Dominant exposure class"
    ) +
    .climniche_theme(base_size = 8.2) +
    ggplot2::theme(
      legend.position = "none"
    )

  p_classes <- ggplot2::ggplot(
    x$classes,
    ggplot2::aes(x = proportion, y = class, fill = class)
  ) +
    ggplot2::geom_col(width = 0.66, colour = "grey25", linewidth = 0.15) +
    ggplot2::geom_text(ggplot2::aes(label = label), hjust = -0.12,
                       size = 2.3) +
    ggplot2::scale_x_continuous(
      labels = function(z) paste0(round(100 * z), "%"),
      limits = c(0, max(x$classes$proportion) * 1.18),
      expand = ggplot2::expansion(mult = c(0, 0.02))
    ) +
    ggplot2::scale_fill_manual(values = class_cols, drop = TRUE) +
    ggplot2::labs(
      title = "(b) Exposure classes",
      x = "Proportion of analysed cells",
      y = NULL
    ) +
    .climniche_theme(base_size = 8.2) +
    ggplot2::theme(legend.position = "none")

  p_vars <- ggplot2::ggplot(
    x$variables,
    ggplot2::aes(x = mean_contribution, y = variable,
                 fill = mean_contribution > 0)
  ) +
    ggplot2::geom_col(width = 0.66, colour = "grey25", linewidth = 0.15) +
    ggplot2::geom_vline(xintercept = 0, linewidth = 0.25,
                        colour = "grey45") +
    ggplot2::scale_fill_manual(
      values = c("TRUE" = "#c65d57", "FALSE" = "#4c78a8"),
      labels = c("TRUE" = "less similar", "FALSE" = "more similar")
    ) +
    ggplot2::labs(
      title = "(c) Variable contributions",
      x = "Mean contribution to niche distance change",
      y = NULL,
      fill = NULL
    ) +
    .climniche_theme(base_size = 8.2) +
    ggplot2::theme(legend.position = "none")

  p_metrics <- ggplot2::ggplot(
    x$metrics,
    ggplot2::aes(x = value)
  ) +
    ggplot2::geom_histogram(bins = 32, fill = "#6f9fbc",
                            colour = "white", linewidth = 0.10) +
    ggplot2::geom_vline(xintercept = 0, linewidth = 0.22,
                        linetype = 2, colour = "grey45") +
    ggplot2::facet_wrap(~metric, scales = "free", nrow = 2) +
    ggplot2::labs(
      title = "(d) Metric distributions",
      x = NULL,
      y = "Number of cells"
    ) +
    .climniche_theme(base_size = 8.2) +
    ggplot2::theme(legend.position = "none")

  plots <- list(
    exposure = p_plane,
    classes = p_classes,
    variables = p_vars,
    metrics = p_metrics
  )
  if (requireNamespace("patchwork", quietly = TRUE)) {
    out <- (p_plane | p_classes) / (p_vars | p_metrics)
    if (!is.null(title)) {
      out <- out + patchwork::plot_annotation(title = title)
    }
    return(out)
  }
  plots
}

#' @export
summary.climniche_fit <- function(object, ...) {
  out <- list(
    n = nrow(object$current),
    p = ncol(object$current),
    occupied_n = length(object$occupied),
    boundary_quantile = object$boundary_quantile,
    boundary_value = object$boundary_value,
    climate_change_amount = base::summary(object$climate_change_amount),
    niche_distance_change = base::summary(object$niche_distance_change),
    composition_change = base::summary(object$composition_change),
    change_alignment = base::summary(object$change_alignment),
    outside_niche_exceedance = base::summary(object$outside_niche_exceedance),
    classification = table(object$classification)
  )
  class(out) <- "summary.climniche_fit"
  out
}

#' @export
plot.climniche_fit <- function(x, type = c("distance", "boundary", "amount",
                                           "classification"), ...) {
  type <- match.arg(type)
  if (type == "distance") {
    graphics::hist(x$niche_distance_change, main = "Niche distance change",
                   xlab = "Niche distance change", ...)
  } else if (type == "boundary") {
    graphics::hist(x$outside_niche_exceedance, main = "Niche boundary exceedance",
                   xlab = "Exceedance", ...)
  } else if (type == "amount") {
    graphics::hist(x$climate_change_amount, main = "Climate change amount",
                   xlab = "Amount", ...)
  } else {
    graphics::barplot(table(x$classification), las = 2,
                      main = "Climate niche change classes", ...)
  }
  invisible(x)
}

.weighted_niche_coordinates <- function(x) {
  ev <- eigen(x$A, symmetric = TRUE)
  keep <- ev$values > sqrt(.Machine$double.eps)
  if (!any(keep)) {
    stop("The niche metric has no positive dimensions.", call. = FALSE)
  }
  transform <- ev$vectors[, keep, drop = FALSE] %*%
    diag(sqrt(ev$values[keep]), nrow = sum(keep))
  current <- sweep(x$current, 2L, x$center, "-") %*% transform
  future <- sweep(x$future, 2L, x$center, "-") %*% transform
  list(current = current, future = future, transform = transform)
}

.niche_diagram_rotation <- function(coords, idx) {
  ref <- coords[idx, , drop = FALSE]
  if (ncol(ref) == 1L) {
    return(matrix(c(1, 0), nrow = 1))
  }
  S <- crossprod(ref) / max(1, nrow(ref))
  ev <- eigen(S, symmetric = TRUE)
  rot <- ev$vectors[, seq_len(min(2L, ncol(ev$vectors))), drop = FALSE]
  if (ncol(rot) == 1L) {
    rot <- cbind(rot, 0)
  }
  rot
}

.hull_df <- function(dat, x = "axis1", y = "axis2") {
  ok <- is.finite(dat[[x]]) & is.finite(dat[[y]])
  dat <- dat[ok, , drop = FALSE]
  if (nrow(dat) < 3L) {
    return(dat[0, , drop = FALSE])
  }
  dat[grDevices::chull(dat[[x]], dat[[y]]), , drop = FALSE]
}

.circle_df <- function(radius, n = 240L) {
  theta <- seq(0, 2 * pi, length.out = n)
  data.frame(
    axis1 = radius * cos(theta),
    axis2 = radius * sin(theta)
  )
}

#' Build data for a niche climate exposure diagram
#'
#' @param x A fitted climniche object.
#' @param scope `"current"` for current occurrence, range or thresholded SDM
#'   cells; `"all"` for all evaluated cells.
#' @param max_arrows Maximum number of current to future arrows to keep.
#' @param seed Random seed used when subsampling arrows.
#'
#' @return A list of data frames used by `plot_climniche_diagram()`.
#' @export
climniche_diagram_data <- function(x, scope = c("current", "all"),
                                   max_arrows = 350L, seed = 1L) {
  if (!inherits(x, "climniche_fit")) {
    stop("x must be a fitted climniche object.", call. = FALSE)
  }
  scope <- match.arg(scope)
  coords <- .weighted_niche_coordinates(x)
  rot <- .niche_diagram_rotation(coords$current, x$occupied)
  current2 <- coords$current %*% rot
  future2 <- coords$future %*% rot
  colnames(current2) <- colnames(future2) <- c("axis1", "axis2")

  tab <- climniche_table(x, scope = scope)
  idx <- tab$cell
  cells <- data.frame(
    cell = idx,
    current_axis1 = current2[idx, 1],
    current_axis2 = current2[idx, 2],
    future_axis1 = future2[idx, 1],
    future_axis2 = future2[idx, 2],
    climate_change_amount = tab$climate_change_amount,
    niche_distance_change = tab$niche_distance_change,
    composition_change = tab$composition_change,
    niche_boundary_exceedance = tab$outside_niche_exceedance,
    class = tab$class,
    mixed_variable_response = tab$mixed_variable_response,
    stringsAsFactors = FALSE
  )
  cells$class <- factor(cells$class, levels = levels(x$classification))
  class_summary <- stats::aggregate(
    cells[, c("current_axis1", "current_axis2", "future_axis1",
              "future_axis2", "climate_change_amount",
              "niche_distance_change", "composition_change",
              "niche_boundary_exceedance")],
    by = list(class = cells$class),
    FUN = mean,
    na.rm = TRUE
  )
  class_counts <- as.data.frame(table(cells$class), stringsAsFactors = FALSE)
  names(class_counts) <- c("class", "count")
  class_summary <- merge(class_summary, class_counts, by = "class",
                         all.x = TRUE, sort = FALSE)
  class_summary$proportion <- class_summary$count / nrow(cells)
  class_summary$class <- factor(class_summary$class,
                                levels = levels(x$classification))

  arrows <- cells
  if (nrow(arrows) > max_arrows) {
    set.seed(seed)
    arrows <- arrows[sample(seq_len(nrow(arrows)), max_arrows), , drop = FALSE]
  }

  reference <- data.frame(axis1 = current2[, 1], axis2 = current2[, 2])
  current_ref <- data.frame(axis1 = current2[x$occupied, 1],
                            axis2 = current2[x$occupied, 2])
  current_radius <- sqrt(rowSums(coords$current[x$occupied, , drop = FALSE]^2))
  current_core <- current_ref[current_radius <= x$boundary_radius, , drop = FALSE]
  if (nrow(current_core) >= 3L) {
    current_ref <- current_core
  }
  future_ref <- data.frame(axis1 = future2[idx, 1], axis2 = future2[idx, 2])
  future_radius <- sqrt(rowSums(coords$future[idx, , drop = FALSE]^2))
  future_cut <- stats::quantile(future_radius, probs = 0.95, na.rm = TRUE,
                                names = FALSE, type = 8)
  future_core <- future_ref[future_radius <= future_cut, , drop = FALSE]
  if (nrow(future_core) >= 3L) {
    future_ref <- future_core
  }

  basis <- diag(ncol(x$current)) %*% coords$transform %*% rot
  if (is.null(colnames(x$current))) {
    var_names <- paste0("var", seq_len(nrow(basis)))
  } else {
    var_names <- colnames(x$current)
  }
  arrow_radius <- stats::quantile(
    sqrt(rowSums(rbind(current2, future2)^2)),
    probs = 0.90,
    na.rm = TRUE,
    names = FALSE
  )
  basis_len <- sqrt(rowSums(basis^2))
  scale <- if (all(basis_len == 0)) 1 else 0.85 * arrow_radius / max(basis_len)
  variables <- data.frame(
    variable = var_names,
    axis1 = basis[, 1] * scale,
    axis2 = basis[, 2] * scale,
    stringsAsFactors = FALSE
  )

  out <- list(
    cells = cells,
    arrows = arrows,
    class_summary = class_summary,
    reference_hull = .hull_df(reference),
    current_hull = .hull_df(current_ref),
    future_hull = .hull_df(future_ref),
    niche_boundary = .circle_df(x$boundary_radius),
    variables = variables,
    center = data.frame(axis1 = 0, axis2 = 0),
    settings = data.frame(
      scope = scope,
      boundary_quantile = x$boundary_quantile,
      boundary_radius = x$boundary_radius,
      n_cells = nrow(cells),
      n_arrows = nrow(arrows)
    )
  )
  class(out) <- "climniche_diagram_data"
  out
}

#' Plot a niche climate exposure diagram
#'
#' @param x A fitted climniche object or data returned by
#'   `climniche_diagram_data()`.
#' @param scope `"current"` for current occurrence, range or thresholded SDM
#'   cells; `"all"` for all evaluated cells.
#' @param type `"summary"` draws class mean arrows; `"sample"` draws sampled
#'   cell arrows and future points.
#' @param max_arrows Maximum number of current to future arrows to draw when
#'   `type = "sample"`.
#' @param seed Random seed used when subsampling arrows.
#' @param show_reference Logical; draw the full analysed environmental domain.
#' @param show_hulls Logical; draw current and future niche hulls.
#' @param boundary_shape Boundary display. `"hull"` draws the empirical
#'   occupied niche polygon, `"circle"` draws a constant niche distance boundary,
#'   and `"none"` suppresses the boundary.
#' @param show_boundary_label Logical; add reference-area explanations below
#'   the exposure-class legend.
#' @param show_points Logical; draw future points when `type = "sample"`.
#' @param show_endpoints Logical; draw class mean future positions when
#'   `type = "summary"`.
#' @param show_center Logical; mark the realised niche centre.
#' @param show_variables Logical; draw environmental variable directions.
#' @param variable_labels Optional named vector replacing variable labels.
#' @param title Optional plot title.
#'
#' @return A ggplot object.
#' @export
plot_climniche_diagram <- function(x, scope = c("current", "all"),
                                   type = c("summary", "sample"),
                                   max_arrows = 350L, seed = 1L,
                                   show_reference = FALSE,
                                   show_hulls = TRUE,
                                   boundary_shape = c("hull", "circle", "none"),
                                   show_boundary_label = TRUE,
                                   show_points = NULL,
                                   show_endpoints = FALSE,
                                   show_center = FALSE,
                                   show_variables = FALSE,
                                   variable_labels = NULL,
                                   title = NULL) {
  .need_ggplot2()
  type <- match.arg(type)
  boundary_shape <- match.arg(boundary_shape)
  if (!inherits(x, "climniche_diagram_data")) {
    x <- climniche_diagram_data(x, scope = scope, max_arrows = max_arrows,
                                seed = seed)
  }
  if (is.null(show_points)) {
    show_points <- identical(type, "sample")
  }
  if (!is.null(variable_labels)) {
    idx <- match(x$variables$variable, names(variable_labels))
    replace <- !is.na(idx)
    x$variables$variable[replace] <- unname(variable_labels[idx[replace]])
  }
  cell_cols <- .class_colours()
  p <- ggplot2::ggplot()
  if (show_reference && nrow(x$reference_hull) > 2L) {
    p <- p + ggplot2::geom_polygon(
      data = x$reference_hull,
      ggplot2::aes(x = axis1, y = axis2),
      fill = "#f5f5f5", colour = "#8a8a8a", linewidth = 0.30
    )
  }
  if (identical(boundary_shape, "circle")) {
    p <- p + ggplot2::geom_path(
      data = x$niche_boundary,
      ggplot2::aes(x = axis1, y = axis2,
                   linetype = "95% current niche distance"),
      colour = "#333333", linewidth = 0.35
    )
  } else if (identical(boundary_shape, "hull") && nrow(x$current_hull) > 2L) {
    p <- p + ggplot2::geom_polygon(
      data = x$current_hull,
      ggplot2::aes(x = axis1, y = axis2,
                   linetype = "95% current niche area"),
      fill = "#9a9a9a", colour = "#333333", alpha = 0.34,
      linewidth = 0.30
    )
  }
  if (show_hulls) {
    if (!identical(boundary_shape, "hull")) {
      p <- p + ggplot2::geom_polygon(
        data = x$current_hull,
        ggplot2::aes(x = axis1, y = axis2,
                     linetype = "current niche area"),
        fill = "#8f8f8f", colour = "#333333", alpha = 0.45,
        linewidth = 0.30
      )
    }
    p <- p + ggplot2::geom_polygon(
      data = x$future_hull,
      ggplot2::aes(x = axis1, y = axis2,
                   linetype = "future climate area"),
      fill = NA, colour = "#1b6f6a", linewidth = 0.35
    )
  }
  if (identical(type, "sample")) {
    p <- p + ggplot2::geom_segment(
        data = x$arrows,
        ggplot2::aes(x = current_axis1, y = current_axis2,
                     xend = future_axis1, yend = future_axis2,
                     colour = class),
        linewidth = 0.22, alpha = 0.32,
        arrow = grid::arrow(length = grid::unit(1.0, "mm"), type = "closed")
      )
  } else {
    p <- p +
      ggplot2::geom_segment(
        data = x$class_summary,
        ggplot2::aes(x = current_axis1, y = current_axis2,
                     xend = future_axis1, yend = future_axis2,
                     colour = class),
        linewidth = 0.75, alpha = 0.95,
        arrow = grid::arrow(length = grid::unit(2.0, "mm"), type = "closed")
      )
    if (show_endpoints) {
      p <- p +
        ggplot2::geom_point(
          data = x$class_summary,
          ggplot2::aes(x = future_axis1, y = future_axis2, colour = class),
          size = 1.8,
          alpha = 0.9
        )
    }
  }
  if (show_points) {
    p <- p + ggplot2::geom_point(
      data = x$cells,
      ggplot2::aes(x = future_axis1, y = future_axis2, colour = class),
      size = 0.65, alpha = 0.35
    )
  }
  if (show_center) {
    p <- p + ggplot2::geom_point(
      data = x$center,
      ggplot2::aes(x = axis1, y = axis2),
      shape = 3, colour = "black", size = 2.5, stroke = 0.45
    )
  }
  p <- p +
    ggplot2::scale_colour_manual(values = cell_cols, drop = TRUE) +
    ggplot2::scale_linetype_manual(
      values = c(
        "95% current niche area" = "solid",
        "95% current niche distance" = "dashed",
        "current niche area" = "solid",
        "future climate area" = "solid"
      ),
      breaks = c(
        "95% current niche area",
        "95% current niche distance",
        "current niche area",
        "future climate area"
      ),
      name = "Reference areas"
    ) +
    ggplot2::guides(
      colour = ggplot2::guide_legend(
        order = 1,
        override.aes = list(alpha = 1, size = 1.8, linewidth = 0.5)
      ),
      linetype = if (show_boundary_label) ggplot2::guide_legend(order = 2) else "none"
    ) +
    ggplot2::coord_equal() +
    ggplot2::labs(
      x = "Niche climate axis 1",
      y = "Niche climate axis 2",
      colour = "Exposure class",
      title = title
    ) +
    .climniche_theme(base_size = 8.5) +
    ggplot2::theme(
      panel.grid.major = ggplot2::element_line(colour = "#e1e1e1",
                                               linewidth = 0.15),
      legend.key.height = grid::unit(4.6, "mm")
    )

  if (show_variables && nrow(x$variables) > 0L) {
    p <- p +
      ggplot2::geom_segment(
        data = x$variables,
        ggplot2::aes(x = 0, y = 0, xend = axis1, yend = axis2),
        inherit.aes = FALSE,
        colour = "#2f2f2f", linewidth = 0.3,
        arrow = grid::arrow(length = grid::unit(1.35, "mm"), type = "closed")
      ) +
      ggplot2::geom_text(
        data = x$variables,
        ggplot2::aes(x = axis1, y = axis2, label = variable),
        inherit.aes = FALSE,
        size = 2.3, colour = "#2f2f2f", hjust = -0.08, vjust = -0.08
      )
  }
  p
}

.as_numeric_matrix <- function(x, arg = "x") {
  if (is.data.frame(x)) {
    x <- as.matrix(x)
  }
  if (!is.matrix(x) || !is.numeric(x)) {
    stop(arg, " must be a numeric matrix or data frame.", call. = FALSE)
  }
  if (any(!is.finite(x))) {
    stop(arg, " contains non-finite values; handle NA/Inf before calling climniche.", call. = FALSE)
  }
  x
}

.occupied_index <- function(occupied, n, threshold = 0) {
  if (is.null(occupied)) {
    return(seq_len(n))
  }
  if (is.logical(occupied)) {
    if (length(occupied) != n) {
      stop("occupied must have length equal to nrow(current).", call. = FALSE)
    }
    return(which(occupied))
  }
  if (is.numeric(occupied)) {
    if (length(occupied) == n) {
      idx <- which(!is.na(occupied) & occupied > threshold)
      if (!length(idx)) {
        stop("No occupied rows found after applying occupied_threshold.",
             call. = FALSE)
      }
      return(idx)
    }
    occupied <- as.integer(occupied)
    if (any(occupied < 1L | occupied > n)) {
      stop("occupied indices must be between 1 and nrow(current).", call. = FALSE)
    }
    return(unique(occupied))
  }
  stop("occupied must be NULL, a logical vector, or integer indices.", call. = FALSE)
}

.quad_form_rows <- function(x, A) {
  rowSums((x %*% A) * x)
}

.validate_niche_metric <- function(A, p, arg = "A",
                                   tol = sqrt(.Machine$double.eps)) {
  A <- as.matrix(A)
  if (!is.numeric(A) || any(!is.finite(A))) {
    stop(arg, " must be a finite numeric matrix.", call. = FALSE)
  }
  if (!identical(dim(A), c(p, p))) {
    stop(arg, " must be a square matrix with dimension ncol(current).",
         call. = FALSE)
  }
  sym_tol <- tol * max(1, max(abs(A)))
  if (max(abs(A - t(A))) > sym_tol) {
    stop(arg, " must be symmetric.", call. = FALSE)
  }
  A <- (A + t(A)) / 2
  ev <- eigen(A, symmetric = TRUE, only.values = TRUE)$values
  ev_tol <- tol * max(1, max(abs(ev)))
  if (min(ev) < -ev_tol) {
    stop(arg, " must be positive semidefinite.", call. = FALSE)
  }
  if (max(ev) <= ev_tol) {
    stop(arg, " must contain at least one positive dimension.",
         call. = FALSE)
  }
  A
}

.standardize_pair <- function(current, future, scale, global_mean, global_sd) {
  if (!scale) {
    return(list(current = current, future = future, center = rep(0, ncol(current)),
                scale = rep(1, ncol(current))))
  }
  if (is.null(global_mean)) {
    global_mean <- colMeans(current)
  }
  if (is.null(global_sd)) {
    global_sd <- apply(current, 2L, stats::sd)
  }
  if (any(global_sd <= 0)) {
    stop("All climate variables must have positive standard deviation.", call. = FALSE)
  }
  list(
    current = sweep(sweep(current, 2L, global_mean, "-"), 2L, global_sd, "/"),
    future = sweep(sweep(future, 2L, global_mean, "-"), 2L, global_sd, "/"),
    center = global_mean,
    scale = global_sd
  )
}

.extract_slot <- function(object, name) {
  if (is.null(object)) {
    return(NULL)
  }
  if (methods::is(object, "S4") && name %in% methods::slotNames(object)) {
    return(methods::slot(object, name))
  }
  if (is.list(object) && !is.null(object[[name]])) {
    return(object[[name]])
  }
  NULL
}

.metric_label <- function(metric) {
  switch(metric,
         climate_change_amount = "Climate change amount",
         niche_distance_change = "Niche distance change",
         composition_change = "Composition change",
         change_alignment = "Change alignment",
         outside_niche_exceedance = "Niche boundary exceedance",
         metric)
}

.need_ggplot2 <- function() {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 is required for this plot.", call. = FALSE)
  }
}

.need_raster <- function() {
  if (!requireNamespace("raster", quietly = TRUE)) {
    stop("raster is required for raster plotting.", call. = FALSE)
  }
}

.need_terra <- function() {
  if (!requireNamespace("terra", quietly = TRUE)) {
    stop("terra is required for terra raster plotting.", call. = FALSE)
  }
}

.is_raster <- function(x) {
  methods::is(x, "Raster")
}

.is_spatraster <- function(x) {
  inherits(x, "SpatRaster")
}

.spatial_df <- function(x, value = "value") {
  if (.is_raster(x)) {
    return(.raster_df(x, value = value))
  }
  if (.is_spatraster(x)) {
    return(.terra_df(x, value = value))
  }
  stop("x must be a RasterLayer or terra SpatRaster.", call. = FALSE)
}

.raster_df <- function(x, value = "value") {
  .need_raster()
  pts <- raster::rasterToPoints(x)
  pts <- as.data.frame(pts)
  names(pts) <- c("x", "y", value)
  pts
}

.terra_df <- function(x, value = "value") {
  .need_terra()
  if (terra::nlyr(x) != 1L) {
    stop("Only one-layer SpatRaster objects can be plotted.", call. = FALSE)
  }
  pts <- terra::as.data.frame(x, xy = TRUE, na.rm = FALSE)
  names(pts)[seq_len(3L)] <- c("x", "y", value)
  pts <- pts[!is.na(pts[[value]]), c("x", "y", value), drop = FALSE]
  pts
}

.spatial_res <- function(x) {
  if (.is_raster(x)) {
    .need_raster()
    return(raster::res(x))
  }
  if (.is_spatraster(x)) {
    .need_terra()
    return(terra::res(x))
  }
  stop("x must be a RasterLayer or terra SpatRaster.", call. = FALSE)
}

.mask_to_occupied <- function(x, occupied, occupied_threshold = 0) {
  if (.is_raster(x)) {
    .need_raster()
    if (!.is_raster(occupied)) {
      stop("occupied must be a RasterLayer when x is a RasterLayer.",
           call. = FALSE)
    }
    if (!raster::compareRaster(x, occupied, stopiffalse = FALSE)) {
      stop("occupied raster must match x geometry.", call. = FALSE)
    }
    mask <- raster::raster(occupied)
    values <- raster::getValues(occupied)
    raster::values(mask) <- ifelse(!is.na(values) & values > occupied_threshold,
                                   1, NA_real_)
    return(raster::mask(x, mask))
  }
  if (.is_spatraster(x)) {
    .need_terra()
    if (!.is_spatraster(occupied)) {
      stop("occupied must be a SpatRaster when x is a SpatRaster.",
           call. = FALSE)
    }
    if (terra::nlyr(occupied) != 1L) {
      stop("occupied must have one layer.", call. = FALSE)
    }
    if (!terra::compareGeom(x, occupied, stopOnError = FALSE)) {
      stop("occupied raster must match x geometry.", call. = FALSE)
    }
    mask <- occupied
    values <- terra::values(mask)[, 1]
    terra::values(mask) <- ifelse(!is.na(values) & values > occupied_threshold,
                                  1, NA_real_)
    return(terra::mask(x, mask))
  }
  stop("x must be a RasterLayer or terra SpatRaster.", call. = FALSE)
}

.occupied_df <- function(occupied, occupied_threshold = 0) {
  if (is.null(occupied)) {
    return(NULL)
  }
  pts <- .spatial_df(occupied, value = "occupied")
  pts[!is.na(pts$occupied) & pts$occupied > occupied_threshold, ,
      drop = FALSE]
}

.crop_df <- function(dat, pad = 0.02) {
  xr <- range(dat$x, na.rm = TRUE)
  yr <- range(dat$y, na.rm = TRUE)
  dx <- diff(xr) * pad
  dy <- diff(yr) * pad
  list(xlim = c(xr[1] - dx, xr[2] + dx), ylim = c(yr[1] - dy, yr[2] + dy))
}

.plot_title <- function(title, default) {
  if (isFALSE(title)) {
    NULL
  } else if (is.null(title)) {
    default
  } else {
    title
  }
}

.class_colours <- function() {
  c(
    "little climate niche change" = "#f0f0f0",
    "closer to current niche" = "#4c78a8",
    "farther from current niche" = "#c65d57",
    "outside current niche boundary" = "#d99b45",
    "changed composition, similar distance" = "#5f9e8f"
  )
}

.climniche_theme <- function(base_size = 8.5) {
  ggplot2::theme_classic(base_size = base_size) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", hjust = 0, size = base_size + 0.2),
      plot.subtitle = ggplot2::element_text(colour = "grey35", size = base_size - 0.8),
      axis.text = ggplot2::element_text(colour = "grey30", size = base_size - 1.4),
      axis.title = ggplot2::element_text(colour = "grey15", size = base_size - 0.5),
      axis.line = ggplot2::element_line(linewidth = 0.25),
      axis.ticks = ggplot2::element_line(linewidth = 0.25),
      legend.position = "right",
      legend.text = ggplot2::element_text(size = base_size - 1.5),
      legend.title = ggplot2::element_text(size = base_size - 1.2),
      legend.key.height = grid::unit(4.6, "mm"),
      strip.background = ggplot2::element_blank(),
      strip.text = ggplot2::element_text(face = "bold", size = base_size - 0.4)
    )
}

.map_theme <- function() {
  ggplot2::theme_classic(base_size = 8.5) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", hjust = 0, size = 8.7),
      axis.text = ggplot2::element_text(colour = "grey35", size = 6.4),
      axis.ticks = ggplot2::element_line(colour = "grey70", linewidth = 0.2),
      axis.line = ggplot2::element_line(linewidth = 0.2),
      legend.position = "right",
      legend.text = ggplot2::element_text(size = 6.8),
      legend.title = ggplot2::element_text(size = 7.2),
      legend.key.width = grid::unit(3.4, "mm"),
      legend.key.height = grid::unit(8, "mm")
    )
}

.climniche_plot_data <- function(x, scope = c("current", "all"),
                                 max_points = 5000, seed = 1) {
  scope <- match.arg(scope)
  dat <- climniche_table(x, scope = scope)
  if (nrow(dat) > max_points) {
    set.seed(seed)
    dat <- dat[sample(seq_len(nrow(dat)), max_points), , drop = FALSE]
  }
  dat$class <- factor(dat$class, levels = names(.class_colours()))
  dat
}

#' Plot a climniche raster metric
#'
#' @param x A `climniche_fit` object with raster outputs, a RasterLayer or a
#'   one-layer terra SpatRaster.
#' @param metric Metric to plot when `x` is a `climniche_fit` object.
#' @param occupied Optional current occurrence, range or SDM raster to overlay.
#' @param occupied_only If TRUE, mask the plotted raster to current occurrence
#'   cells.
#' @param occupied_threshold Threshold used when `occupied` contains binary or
#'   continuous values.
#' @param title Optional plot title. Use `FALSE` to suppress it.
#' @param midpoint Midpoint for the niche distance change colour scale.
#'
#' @return A ggplot object.
#' @noRd
.plot_climniche_map <- function(x,
                                metric = c("niche_distance_change",
                                           "outside_niche_exceedance",
                                           "climate_change_amount",
                                           "composition_change",
                                           "change_alignment"),
                                occupied = NULL,
                                occupied_only = FALSE,
                                occupied_threshold = 0,
                                title = NULL,
                                midpoint = 0) {
  .need_ggplot2()
  metric <- match.arg(metric)

  if (inherits(x, "climniche_fit")) {
    if (is.null(x$rasters)) {
      stop("x does not contain raster outputs.", call. = FALSE)
    }
    r <- x$rasters[[metric]]
  } else {
    r <- x
  }
  if (!.is_raster(r) && !.is_spatraster(r)) {
    stop("x must be a climniche object with raster outputs, a RasterLayer or a SpatRaster.",
         call. = FALSE)
  }
  if (occupied_only) {
    if (is.null(occupied)) {
      stop("occupied must be supplied when occupied_only = TRUE.",
           call. = FALSE)
    }
    r <- .mask_to_occupied(r, occupied,
                           occupied_threshold = occupied_threshold)
  }

  dat <- .spatial_df(r, value = "value")
  occ <- .occupied_df(occupied, occupied_threshold = occupied_threshold)
  lim <- .crop_df(dat)
  ttl <- .plot_title(title, .metric_label(metric))
  cell_size <- .spatial_res(r)

  p <- ggplot2::ggplot(dat, ggplot2::aes(x = x, y = y, fill = value)) +
    ggplot2::geom_tile(width = cell_size[1], height = cell_size[2]) +
    ggplot2::coord_equal(xlim = lim$xlim, ylim = lim$ylim, expand = FALSE) +
    ggplot2::labs(x = NULL, y = NULL, fill = .metric_label(metric),
                  title = ttl) +
    .map_theme()

  if (identical(metric, "niche_distance_change")) {
    p <- p + ggplot2::scale_fill_gradient2(
      low = "#4c78a8", mid = "white", high = "#c65d57",
      midpoint = midpoint, na.value = NA
    )
  } else if (identical(metric, "change_alignment")) {
    p <- p + ggplot2::scale_fill_gradient2(
      low = "#4c78a8", mid = "white", high = "#c65d57",
      midpoint = 0, limits = c(-1, 1), na.value = NA
    )
  } else if (identical(metric, "outside_niche_exceedance")) {
    p <- p + ggplot2::scale_fill_gradientn(
      colours = c("white", "#f2d7a0", "#d99b45", "#7a3b20"),
      na.value = NA
    )
  } else if (identical(metric, "composition_change")) {
    p <- p + ggplot2::scale_fill_gradientn(
      colours = c("#f7fcf5", "#b7e0c2", "#5f9e8f", "#1b5e50"),
      na.value = NA
    )
  } else {
    p <- p + ggplot2::scale_fill_gradientn(
      colours = c("#f7fbff", "#aac7df", "#5b91bd", "#1f4f78"),
      na.value = NA
    )
  }

  if (!occupied_only && !is.null(occ) && nrow(occ) > 0) {
    p <- p + ggplot2::geom_point(
      data = occ,
      ggplot2::aes(x = x, y = y),
      inherit.aes = FALSE,
      size = 0.05,
      colour = "grey10",
      alpha = 0.14
    )
  }
  p
}

#' Plot the climniche exposure plane
#'
#' @param x A fitted `climniche_fit` object.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#' @param max_points Maximum number of points to draw.
#' @param seed Random seed used when subsampling.
#' @param title Optional plot title.
#'
#' @return A ggplot object.
#' @noRd
.plot_climniche_exposure <- function(x, scope = c("current", "all"),
                                     max_points = 6000, seed = 1,
                                     title = NULL) {
  .need_ggplot2()
  if (!inherits(x, "climniche_fit")) {
    stop("x must be a fitted climniche object.", call. = FALSE)
  }
  dat <- .climniche_plot_data(x, scope = scope, max_points = max_points,
                              seed = seed)
  dat$x_value <- dat$climate_change_amount
  dat$y_value <- dat$niche_distance_change
  ttl <- .plot_title(title, "Niche climate exposure")

  ggplot2::ggplot(
    dat,
    ggplot2::aes(x = x_value, y = y_value, colour = class)
  ) +
    ggplot2::geom_hline(yintercept = 0, colour = "grey45",
                        linewidth = 0.3, linetype = 2) +
    ggplot2::geom_point(alpha = 0.42, size = 0.65) +
    ggplot2::scale_colour_manual(values = .class_colours(), drop = FALSE) +
    ggplot2::labs(
      x = "Climate change amount",
      y = "Niche distance change",
      colour = NULL,
      title = ttl,
      subtitle = "Positive values indicate divergence from the current realised climatic niche"
    ) +
    .climniche_theme()
}

#' Plot climniche class proportions
#'
#' @param x A fitted `climniche_fit` object.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#' @param title Optional plot title.
#'
#' @return A ggplot object.
#' @noRd
.plot_climniche_class_summary <- function(x, scope = c("current", "all"),
                                          title = NULL) {
  .need_ggplot2()
  if (!inherits(x, "climniche_fit")) {
    stop("x must be a fitted climniche object.", call. = FALSE)
  }
  scope <- match.arg(scope)
  tab <- climniche_table(x, scope = scope)
  dat <- as.data.frame(prop.table(table(tab$class)), stringsAsFactors = FALSE)
  names(dat) <- c("class", "proportion")
  dat$class <- factor(dat$class, levels = names(.class_colours()))
  dat <- dat[order(dat$proportion), , drop = FALSE]
  dat$prop <- dat$proportion
  ttl <- .plot_title(title, "climniche class proportions")

  ggplot2::ggplot(dat, ggplot2::aes(x = class, y = prop, fill = class)) +
    ggplot2::geom_col(width = 0.68, colour = "grey25", linewidth = 0.18) +
    ggplot2::coord_flip() +
    ggplot2::scale_y_continuous(labels = function(z) paste0(round(100 * z), "%")) +
    ggplot2::scale_fill_manual(values = .class_colours(), drop = FALSE) +
    ggplot2::labs(x = NULL, y = "Proportion of analysed cells", title = ttl) +
    .climniche_theme() +
    ggplot2::theme(legend.position = "none")
}

#' Plot a climniche metric distribution
#'
#' @param x A fitted `climniche_fit` object.
#' @param metric One of `"climate_change_amount"`, `"niche_distance_change"`,
#'   `"outside_niche_exceedance"` or `"composition_change"`.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#' @param title Optional plot title.
#'
#' @return A ggplot object.
#' @noRd
.plot_climniche_distribution <- function(x,
                                         metric = c("niche_distance_change",
                                                    "climate_change_amount",
                                                    "outside_niche_exceedance",
                                                    "composition_change"),
                                         scope = c("current", "all"),
                                         title = NULL) {
  .need_ggplot2()
  metric <- match.arg(metric)
  scope <- match.arg(scope)
  tab <- climniche_table(x, scope = scope)
  tab$value <- tab[[metric]]
  ttl <- .plot_title(title, .metric_label(metric))

  ggplot2::ggplot(tab, ggplot2::aes(x = value)) +
    ggplot2::geom_histogram(bins = 36, fill = "#5b91bd",
                            colour = "white", linewidth = 0.15) +
    ggplot2::geom_vline(xintercept = 0, colour = "grey35",
                        linetype = 2, linewidth = 0.3) +
    ggplot2::labs(x = .metric_label(metric), y = "Number of cells",
                  title = ttl) +
    .climniche_theme() +
    ggplot2::theme(legend.position = "none")
}

#' Plot a climniche report figure
#'
#' @param x A fitted `climniche_fit` object.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#'
#' @return A patchwork object when `patchwork` is installed, otherwise a named
#'   list of ggplot objects.
#' @noRd
.plot_climniche_report <- function(x, scope = c("current", "all")) {
  scope <- match.arg(scope)
  plot_climniche_showcase(x, scope = scope)
}

#' Plot projected climate change classes
#'
#' @param x A `climniche_fit` object with raster outputs.
#' @param occupied Optional occupied-cell RasterLayer or SpatRaster to overlay.
#' @param occupied_only If TRUE, mask the plotted classes to occupied cells.
#' @param occupied_threshold Threshold used when `occupied` contains binary or
#'   continuous values.
#' @param title Optional plot title. Use `FALSE` to suppress it.
#'
#' @return A ggplot object.
#' @noRd
.plot_climniche_classes <- function(x, occupied = NULL, occupied_only = FALSE,
                                    occupied_threshold = 0,
                                    title = NULL) {
  .need_ggplot2()
  if (!inherits(x, "climniche_fit") || is.null(x$rasters$classification)) {
    stop("x must be a climniche object with a classification raster.",
         call. = FALSE)
  }
  r <- x$rasters$classification
  if (occupied_only) {
    if (is.null(occupied)) {
      stop("occupied must be supplied when occupied_only = TRUE.",
           call. = FALSE)
    }
    r <- .mask_to_occupied(r, occupied,
                           occupied_threshold = occupied_threshold)
  }
  dat <- .spatial_df(r, value = "class_id")
  lookup <- x$class_lookup
  dat$class <- lookup$class[match(dat$class_id, lookup$id)]
  dat$class <- factor(dat$class, levels = lookup$class)
  occ <- .occupied_df(occupied, occupied_threshold = occupied_threshold)
  lim <- .crop_df(dat)
  cell_size <- .spatial_res(r)

  p <- ggplot2::ggplot(dat, ggplot2::aes(x = x, y = y, fill = class)) +
    ggplot2::geom_tile(width = cell_size[1], height = cell_size[2]) +
    ggplot2::coord_equal(xlim = lim$xlim, ylim = lim$ylim, expand = FALSE) +
    ggplot2::scale_fill_manual(
      values = .class_colours(),
      na.value = NA
    ) +
    ggplot2::labs(x = NULL, y = NULL, fill = "Exposure class",
                  title = .plot_title(title, "Climate niche change classes")) +
    .map_theme() +
    ggplot2::theme(legend.key.height = grid::unit(4.6, "mm"))

  if (!occupied_only && !is.null(occ) && nrow(occ) > 0) {
    p <- p + ggplot2::geom_point(
      data = occ,
      ggplot2::aes(x = x, y = y),
      inherit.aes = FALSE,
      size = 0.05,
      colour = "grey10",
      alpha = 0.12
    )
  }
  p
}

#' Plot mean variable contribution
#'
#' @param x A `climniche_fit` object.
#' @param occupied_only If TRUE, summarize occupied cells only.
#' @param variable_labels Optional named vector replacing variable labels.
#' @param title Optional plot title. Use `FALSE` to suppress it.
#'
#' @return A ggplot object.
#' @export
plot_climniche_variable_contribution <- function(x, occupied_only = TRUE,
                                                 variable_labels = NULL,
                                                 title = NULL) {
  .need_ggplot2()
  if (!inherits(x, "climniche_fit")) {
    stop("x must be a climniche object.", call. = FALSE)
  }
  idx <- if (occupied_only) x$occupied else seq_len(nrow(x$variable_contribution))
  vals <- colMeans(x$variable_contribution[idx, , drop = FALSE])
  variables <- names(vals)
  if (!is.null(variable_labels)) {
    replace <- match(variables, names(variable_labels))
    hit <- !is.na(replace)
    variables[hit] <- unname(variable_labels[replace[hit]])
  }
  dat <- data.frame(
    variable = factor(variables, levels = variables),
    contribution = as.numeric(vals)
  )

  ggplot2::ggplot(dat, ggplot2::aes(x = variable, y = contribution,
                                    fill = contribution > 0)) +
    ggplot2::geom_col(width = 0.65, colour = "grey25", linewidth = 0.2) +
    ggplot2::geom_hline(yintercept = 0, linewidth = 0.3, colour = "grey35") +
    ggplot2::scale_fill_manual(values = c("TRUE" = "#c65d57",
                                          "FALSE" = "#4c78a8")) +
    ggplot2::labs(x = NULL, y = "Mean contribution",
                  title = .plot_title(title, "Variable contribution")) +
    ggplot2::theme_classic(base_size = 8.5) +
    ggplot2::theme(
      legend.position = "none",
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 7),
      axis.text.y = ggplot2::element_text(size = 7),
      axis.title.y = ggplot2::element_text(size = 8),
      axis.line = ggplot2::element_line(linewidth = 0.2),
      axis.ticks = ggplot2::element_line(linewidth = 0.2)
    )
}

#' Plot mean variable contribution
#'
#' @param x A `climniche_fit` object.
#' @param occupied_only If TRUE, summarize occupied cells only.
#' @param variable_labels Optional named vector replacing variable labels.
#' @param title Optional plot title. Use `FALSE` to suppress it.
#'
#' @return A ggplot object.
#' @export
plot_variable_contribution <- function(x, occupied_only = TRUE,
                                       variable_labels = NULL,
                                       title = NULL) {
  plot_climniche_variable_contribution(
    x = x,
    occupied_only = occupied_only,
    variable_labels = variable_labels,
    title = title
  )
}

#' Simulate a minimal climate niche change experiment
#'
#' @param n Number of climate cells.
#' @param p Number of climate variables.
#' @param seed Random seed.
#' @param rho Pairwise correlation among simulated climate variables.
#' @param prevalence Proportion of background cells treated as true current
#'   occurrence locations under the virtual niche.
#' @param shift Climate change amount imposed in the closer to niche and
#'   farther from niche scenarios.
#'
#' @return A list with current, future_toward, future_away, occupied, center,
#'   sensitivity and A.
#' @export
simulate_climniche <- function(n = 2000, p = 2, seed = 1,
                               rho = 0, prevalence = 0.30,
                               shift = 0.40) {
  set.seed(seed)
  if (rho < -1 / (p - 1) || rho >= 1) {
    stop("rho must define a positive-definite equicorrelation matrix.",
         call. = FALSE)
  }
  if (prevalence <= 0 || prevalence >= 1) {
    stop("prevalence must be between 0 and 1.", call. = FALSE)
  }
  if (shift <= 0) {
    stop("shift must be positive.", call. = FALSE)
  }

  center <- rep(0, p)
  sensitivity <- seq(1, 2, length.out = p)
  A <- niche_metric(sensitivity = sensitivity)

  Sigma <- matrix(rho, p, p)
  diag(Sigma) <- 1
  L <- chol(Sigma)
  current <- matrix(stats::rnorm(n * p), ncol = p) %*% L
  colnames(current) <- paste0("bio", seq_len(p))
  psi <- niche_potential(current, center = center, A = A)
  occupied <- psi <= stats::quantile(psi, prevalence)

  radial <- sweep(current, 2L, center, "-")
  radial_norm <- sqrt(pmax(.quad_form_rows(radial, A), .Machine$double.eps))
  unit <- radial / radial_norm
  step <- shift * unit
  future_toward <- current - step
  future_away <- current + step

  list(
    current = current,
    future_toward = future_toward,
    future_away = future_away,
    occupied = occupied,
    center = center,
    sensitivity = sensitivity,
    A = A
  )
}

#' Build a sensitivity-weighted niche metric
#'
#' @param sensitivity Numeric vector of climate-variable sensitivity weights.
#' @param cnfa Optional CENFA `cnfa` object or list with `sf`, `co`, and `eig`.
#' @param type Metric type. `"diag"` uses variable-level sensitivity weights.
#'   `"factor"` uses a factor metric when CNFA factor coordinates are available.
#'
#' @return A positive semi-definite matrix.
#' @export
niche_metric <- function(sensitivity = NULL, cnfa = NULL,
                         type = c("diag", "factor")) {
  type <- match.arg(type)
  if (is.null(sensitivity)) {
    sensitivity <- .extract_slot(cnfa, "sf")
  }
  if (is.null(sensitivity)) {
    stop("Provide sensitivity or a cnfa object with an sf slot.", call. = FALSE)
  }
  sensitivity <- as.numeric(sensitivity)
  if (any(!is.finite(sensitivity)) || any(sensitivity < 0)) {
    stop("sensitivity must contain finite non-negative values.", call. = FALSE)
  }
  if (all(sensitivity == 0)) {
    stop("At least one sensitivity weight must be positive.", call. = FALSE)
  }

  if (type == "factor") {
    U <- .extract_slot(cnfa, "co")
    rho <- .extract_slot(cnfa, "eig")
    if (!is.null(U) && !is.null(rho)) {
      U <- as.matrix(U)
      rho <- as.numeric(rho)
      if (any(!is.finite(U))) {
        stop("CNFA factor coordinates must be finite.", call. = FALSE)
      }
      if (any(!is.finite(rho)) || any(rho < 0) || mean(rho) <= 0) {
        stop("CNFA factor eigenvalues must be finite non-negative values with positive mean.",
             call. = FALSE)
      }
      if (nrow(U) == length(sensitivity) && ncol(U) == length(rho)) {
        rho <- rho / mean(rho)
        A <- U %*% diag(rho, nrow = length(rho)) %*% t(U)
        return(.validate_niche_metric(A, length(sensitivity), "A"))
      }
      warning("CNFA factor dimensions are inconsistent; falling back to diag metric.",
              call. = FALSE)
    } else {
      warning("CNFA factor slots not available; falling back to diag metric.",
              call. = FALSE)
    }
  }

  w <- sensitivity / mean(sensitivity)
  .validate_niche_metric(diag(w, nrow = length(w)), length(w), "A")
}

#' Extract a tidy climniche table
#'
#' @param x A fitted `climniche_fit` object.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#'
#' @return A data frame with one row per evaluated cell.
#' @export
climniche_table <- function(x, scope = c("current", "all")) {
  if (!inherits(x, "climniche_fit")) {
    stop("x must be a fitted climniche object.", call. = FALSE)
  }
  scope <- match.arg(scope)
  idx <- if (scope == "current") x$occupied else seq_len(nrow(x$current))
  mixed <- if (!is.null(x$mixed_variable_response)) {
    x$mixed_variable_response[idx]
  } else {
    rep(FALSE, length(idx))
  }

  data.frame(
    cell = idx,
    current_niche_distance = x$niche_radius_current[idx],
    future_niche_distance = x$niche_radius_future[idx],
    climate_change_amount = x$climate_change_amount[idx],
    niche_distance_change = x$niche_distance_change[idx],
    composition_change = x$composition_change[idx],
    change_alignment = x$change_alignment[idx],
    outside_niche_exceedance = x$outside_niche_exceedance[idx],
    current_niche_percentile = x$niche_percentile$current[idx],
    future_niche_percentile = x$niche_percentile$future[idx],
    percentile_change = x$niche_percentile$delta[idx],
    class = x$classification[idx],
    mixed_variable_response = mixed,
    stringsAsFactors = FALSE
  )
}

#' Summarise climniche results
#'
#' @param x A fitted `climniche_fit` object.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#'
#' @return A one-row data frame with key report metrics.
#' @export
climniche_summary <- function(x, scope = c("current", "all")) {
  tab <- climniche_table(x, scope = scope)
  class_prop <- prop.table(table(tab$class))
  get_prop <- function(label) {
    if (label %in% names(class_prop)) unname(class_prop[[label]]) else 0
  }

  data.frame(
    scope = match.arg(scope),
    n = nrow(tab),
    boundary_quantile = x$boundary_quantile,
    boundary_distance = x$boundary_radius,
    mean_climate_change_amount = mean(tab$climate_change_amount, na.rm = TRUE),
    median_climate_change_amount = stats::median(tab$climate_change_amount,
                                                 na.rm = TRUE),
    q90_climate_change_amount = as.numeric(stats::quantile(
      tab$climate_change_amount, 0.90, na.rm = TRUE, names = FALSE
    )),
    mean_niche_distance_change = mean(tab$niche_distance_change, na.rm = TRUE),
    median_niche_distance_change = stats::median(tab$niche_distance_change,
                                                 na.rm = TRUE),
    prop_niche_convergence = get_prop("closer to current niche"),
    prop_niche_divergence = get_prop("farther from current niche"),
    prop_niche_exceedance = get_prop("outside current niche boundary"),
    prop_reconfiguration = get_prop("changed composition, similar distance"),
    prop_stable = get_prop("little climate niche change"),
    prop_mixed_variable_response = mean(tab$mixed_variable_response,
                                        na.rm = TRUE),
    prop_outside_niche = mean(tab$outside_niche_exceedance > 0, na.rm = TRUE),
    mean_outside_niche_exceedance = mean(tab$outside_niche_exceedance,
                                         na.rm = TRUE),
    stringsAsFactors = FALSE
  )
}

#' Build a climniche report
#'
#' @param x A fitted `climniche_fit` object.
#' @param species Optional species name used in printed reports.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#' @param top_variables Number of variable contributions to show.
#'
#' @return An object of class `climniche_report`.
#' @export
climniche_report <- function(x, species = NULL, scope = c("current", "all"),
                             top_variables = 5) {
  if (!inherits(x, "climniche_fit")) {
    stop("x must be a fitted climniche object.", call. = FALSE)
  }
  scope <- match.arg(scope)
  tab <- climniche_table(x, scope = scope)
  summ <- climniche_summary(x, scope = scope)
  idx <- if (scope == "current") x$occupied else seq_len(nrow(x$current))

  cls <- as.data.frame(prop.table(table(tab$class)), stringsAsFactors = FALSE)
  names(cls) <- c("class", "proportion")
  cls$count <- as.integer(table(tab$class)[as.character(cls$class)])
  cls <- cls[order(cls$proportion, decreasing = TRUE), , drop = FALSE]

  vals <- colMeans(x$variable_contribution[idx, , drop = FALSE], na.rm = TRUE)
  var_tab <- data.frame(
    variable = names(vals),
    mean_contribution = as.numeric(vals),
    abs_mean_contribution = abs(as.numeric(vals)),
    interpretation = ifelse(vals > 0,
                            "less similar to current niche",
                            "more similar to current niche"),
    stringsAsFactors = FALSE
  )
  var_tab <- var_tab[order(var_tab$abs_mean_contribution, decreasing = TRUE),
                     , drop = FALSE]
  var_tab <- utils::head(var_tab, top_variables)

  dominant <- if (nrow(cls)) as.character(cls$class[1]) else NA_character_
  direction <- if (summ$mean_niche_distance_change > 0) {
    "less similar to the current realised climatic niche on average"
  } else if (summ$mean_niche_distance_change < 0) {
    "more similar to the current realised climatic niche on average"
  } else {
    "similar in average distance to the current realised climatic niche"
  }

  interpretation <- c(
    paste0("The analysed cells are projected to become ", direction, "."),
    paste0("The most frequent climniche class is '", dominant, "'."),
    paste0(round(100 * summ$prop_mixed_variable_response, 1),
           "% of analysed cells show mixed variable contributions without ",
           "crossing the niche boundary."),
    paste0(round(100 * summ$prop_outside_niche, 1),
           "% of analysed cells exceed the empirical niche boundary at q = ",
           summ$boundary_quantile, ".")
  )

  out <- list(
    species = species,
    scope = scope,
    settings = data.frame(
      boundary_quantile = x$boundary_quantile,
      boundary_distance = x$boundary_radius,
      n_variables = ncol(x$current),
      n_cells = nrow(tab),
      stringsAsFactors = FALSE
    ),
    summary = summ,
    class_summary = cls,
    top_variables = var_tab,
    interpretation = interpretation,
    table = tab
  )
  class(out) <- "climniche_report"
  out
}

#' @export
print.climniche_report <- function(x, ...) {
  title <- if (is.null(x$species)) {
    "climniche report"
  } else {
    paste0("climniche report: ", x$species)
  }
  cat(title, "\n", sep = "")
  cat(strrep("-", nchar(title)), "\n", sep = "")
  cat("Scope: ", x$scope, "\n", sep = "")
  cat("Cells: ", x$settings$n_cells, "\n", sep = "")
  cat("Boundary quantile: ", x$settings$boundary_quantile, "\n\n", sep = "")

  cat("Interpretation\n")
  for (line in x$interpretation) {
    cat("- ", line, "\n", sep = "")
  }

  cat("\nSummary\n")
  print(x$summary, row.names = FALSE)

  cat("\nClass proportions\n")
  print(x$class_summary, row.names = FALSE)

  cat("\nTop variable contributions\n")
  print(x$top_variables, row.names = FALSE)
  invisible(x)
}

#' Write a climniche report to Markdown
#'
#' @param report An object returned by [climniche_report()].
#' @param file Output Markdown file.
#'
#' @return Invisibly returns `file`.
#' @export
write_climniche_report <- function(report, file) {
  if (!inherits(report, "climniche_report")) {
    stop("report must be produced by climniche_report().", call. = FALSE)
  }
  title <- if (is.null(report$species)) {
    "Niche climate exposure report"
  } else {
    paste0("Niche climate exposure report: ",
           report$species)
  }

  fmt_row <- function(dat) {
    paste(utils::capture.output(print(dat, row.names = FALSE)), collapse = "\n")
  }

  lines <- c(
    paste0("# ", title),
    "",
    "## Interpretation",
    paste0("- ", report$interpretation),
    "",
    "## Settings",
    "```text",
    fmt_row(report$settings),
    "```",
    "",
    "## Summary",
    "```text",
    fmt_row(report$summary),
    "```",
    "",
    "## Class Proportions",
    "```text",
    fmt_row(report$class_summary),
    "```",
    "",
    "## Top Variable Contributions",
    "```text",
    fmt_row(report$top_variables),
    "```",
    "",
    "## Notes",
    paste(
      "climniche is an exposure interpretation. It does not estimate population",
      "growth, dispersal, adaptation, biotic interactions or conservation",
      "priority by itself."
    )
  )
  writeLines(lines, con = file)
  invisible(file)
}

#' Fit niche climate exposure
#'
#' @param current Numeric matrix or data frame of current environmental values.
#' @param future Numeric matrix or data frame of future environmental values.
#' @param occupied NULL, logical vector, row indices, or a numeric vector with
#'   one value per row identifying current occurrence, range, or thresholded SDM
#'   cells.
#' @param occupied_threshold Threshold used when `occupied` contains binary or
#'   continuous values.
#' @param cnfa Optional CENFA model object.
#' @param center Optional realised niche centre in standardised climate space.
#' @param sensitivity Optional environmental sensitivity weights.
#' @param A Optional niche metric matrix.
#' @param metric Metric construction when `A` is not supplied.
#' @param boundary Quantile defining the empirical realised niche boundary.
#' @param scale Logical. If TRUE, standardise current and future values.
#' @param global_mean Optional means used for standardisation.
#' @param global_sd Optional standard deviations used for standardisation.
#' @param conflict_ratio Minimum minority sign contribution share used to mark
#'   mixed variable responses. Set to NULL to disable this flag.
#'
#' @return An object of class `climniche_fit`.
#' @export
fit_climniche <- function(current, future, occupied = NULL,
                          occupied_threshold = 0, cnfa = NULL,
                          center = NULL, sensitivity = NULL, A = NULL,
                          metric = c("diag", "factor"),
                          boundary = 0.95, scale = TRUE,
                          global_mean = NULL, global_sd = NULL,
                          conflict_ratio = 0.25) {
  .fit_climniche_matrix(
    current = current,
    future = future,
    occupied = occupied,
    occupied_threshold = occupied_threshold,
    cnfa = cnfa,
    center = center,
    sensitivity = sensitivity,
    A = A,
    metric = metric,
    boundary = boundary,
    scale = scale,
    global_mean = global_mean,
    global_sd = global_sd,
    conflict_ratio = conflict_ratio
  )
}

#' Fit climniche to raster data
#'
#' @param current Raster* object of current environmental layers.
#' @param future Raster* object of future environmental layers.
#' @param occupied Optional RasterLayer with binary or continuous occurrence,
#'   range, or SDM suitability values.
#' @param occupied_threshold Values greater than this threshold are treated as
#'   current occurrence cells.
#' @param domain Optional RasterLayer limiting cells where exposure is analysed.
#' @param domain_threshold Values greater than this threshold define the domain.
#' @param ... Additional arguments passed to `fit_climniche()`.
#'
#' @return An object of class `climniche_fit` with raster outputs.
#' @export
fit_climniche_raster <- function(current, future, occupied = NULL,
                                 occupied_threshold = 0, domain = NULL,
                                 domain_threshold = 0, ...) {
  .fit_climniche_raster(current = current, future = future,
                        occupied = occupied,
                        occupied_threshold = occupied_threshold,
                        domain = domain,
                        domain_threshold = domain_threshold, ...)
}

#' Fit climniche to terra raster data
#'
#' @param current terra SpatRaster of current environmental layers.
#' @param future terra SpatRaster of future environmental layers.
#' @param occupied Optional one layer SpatRaster with binary or continuous
#'   occurrence, range, or SDM suitability values.
#' @param occupied_threshold Values greater than this threshold are treated as
#'   current occurrence cells.
#' @param domain Optional one layer SpatRaster limiting cells where exposure is
#'   analysed.
#' @param domain_threshold Values greater than this threshold define the domain.
#' @param ... Additional arguments passed to `fit_climniche()`.
#'
#' @return An object of class `climniche_fit` with raster outputs.
#' @export
fit_climniche_terra <- function(current, future, occupied = NULL,
                                occupied_threshold = 0, domain = NULL,
                                domain_threshold = 0, ...) {
  .fit_climniche_terra(current = current, future = future,
                       occupied = occupied,
                       occupied_threshold = occupied_threshold,
                       domain = domain,
                       domain_threshold = domain_threshold, ...)
}

#' Plot a climniche map
#'
#' @param x A fitted climniche object with raster outputs, or a RasterLayer.
#' @param metric Metric to plot.
#' @param occupied Optional current occurrence/range RasterLayer to overlay.
#' @param occupied_only If TRUE, mask the plotted raster to current occurrence
#'   cells.
#' @param occupied_threshold Threshold used when `occupied` contains binary or
#'   continuous values.
#' @param title Optional plot title. Use `FALSE` to suppress it.
#' @param midpoint Midpoint for the niche distance change colour scale.
#'
#' @return A ggplot object.
#' @export
plot_climniche_map <- function(x,
                               metric = c("niche_distance_change",
                                          "outside_niche_exceedance",
                                          "climate_change_amount",
                                          "composition_change",
                                          "change_alignment"),
                               occupied = NULL,
                               occupied_only = FALSE,
                               occupied_threshold = 0,
                               title = NULL,
                               midpoint = 0) {
  metric <- match.arg(metric)
  .plot_climniche_map(x = x, metric = metric, occupied = occupied,
                      occupied_only = occupied_only,
                      occupied_threshold = occupied_threshold,
                      title = title,
                      midpoint = midpoint)
}

#' Plot climniche classes
#'
#' @param x A fitted climniche object with raster outputs.
#' @param occupied Optional current occurrence/range RasterLayer to overlay.
#' @param occupied_only If TRUE, mask the plotted classes to current occurrence
#'   cells.
#' @param occupied_threshold Threshold used when `occupied` contains binary or
#'   continuous values.
#' @param title Optional plot title. Use `FALSE` to suppress it.
#'
#' @return A ggplot object.
#' @export
plot_climniche_classes <- function(x, occupied = NULL, occupied_only = FALSE,
                                   occupied_threshold = 0,
                                   title = NULL) {
  .plot_climniche_classes(x = x, occupied = occupied,
                          occupied_only = occupied_only,
                          occupied_threshold = occupied_threshold,
                          title = title)
}

#' Plot the climniche exposure plane
#'
#' @param x A fitted climniche object.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#' @param max_points Maximum number of points to draw.
#' @param seed Random seed used when subsampling.
#' @param title Optional plot title.
#'
#' @return A ggplot object.
#' @export
plot_climniche_exposure <- function(x, scope = c("current", "all"),
                                    max_points = 6000, seed = 1,
                                    title = NULL) {
  .plot_climniche_exposure(x = x, scope = scope, max_points = max_points,
                           seed = seed, title = title)
}

#' Plot climniche class proportions
#'
#' @param x A fitted climniche object.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#' @param title Optional plot title.
#'
#' @return A ggplot object.
#' @export
plot_climniche_class_summary <- function(x, scope = c("current", "all"),
                                         title = NULL) {
  .plot_climniche_class_summary(x = x, scope = scope, title = title)
}

#' Plot a climniche metric distribution
#'
#' @param x A fitted climniche object.
#' @param metric Metric to plot.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#' @param title Optional plot title.
#'
#' @return A ggplot object.
#' @export
plot_climniche_distribution <- function(x,
                                        metric = c("niche_distance_change",
                                                   "climate_change_amount",
                                                   "outside_niche_exceedance",
                                                   "composition_change"),
                                        scope = c("current", "all"),
                                        title = NULL) {
  metric <- match.arg(metric)
  .plot_climniche_distribution(x = x, metric = metric, scope = scope,
                               title = title)
}

#' Plot a climniche report figure
#'
#' @param x A fitted climniche object.
#' @param scope `"current"` for current occurrence/range cells or `"all"` for
#'   all evaluated cells.
#'
#' @return A patchwork object when `patchwork` is installed, otherwise a named
#'   list of ggplot objects.
#' @export
plot_climniche_report <- function(x, scope = c("current", "all")) {
  .plot_climniche_report(x = x, scope = scope)
}

#' Fit niche climate exposure
#'
#' @param current Numeric matrix or data frame of current climate values.
#' @param future Numeric matrix or data frame of future climate values, with the
#'   same dimensions and variables as `current`.
#' @param occupied NULL, logical vector, row indices, or a numeric vector with
#'   one value per row. Numeric vectors of length `nrow(current)` are treated as
#'   binary or continuous occurrence, range, or SDM suitability values.
#' @param occupied_threshold Threshold used when `occupied` is a numeric vector
#'   with one value per row.
#' @param cnfa Optional CENFA `cnfa` object. When supplied, its `mf` and `sf`
#'   slots are used unless `center`, `sensitivity`, or `A` are provided.
#' @param center Optional realised niche centre in standardized climate space.
#' @param sensitivity Optional climate-variable sensitivity weights.
#' @param A Optional niche metric matrix.
#' @param metric `"diag"` or `"factor"`. Used only when `A` is missing.
#' @param boundary Quantile for the empirical occupied niche boundary.
#' @param scale Logical. If TRUE, standardize current and future values using
#'   means and standard deviations of current values.
#' @param global_mean Optional means used for standardization.
#' @param global_sd Optional standard deviations used for standardization.
#' @param conflict_ratio Minimum minority-sign variable-contribution share used
#'   to mark mixed variable responses. Set to NULL to disable this flag.
#'
#' @return An object of class `climniche_fit`.
#' @noRd
.fit_climniche_matrix <- function(current, future, occupied = NULL,
                                  occupied_threshold = 0,
                                  cnfa = NULL, center = NULL,
                                  sensitivity = NULL, A = NULL,
                                  metric = c("diag", "factor"),
                                  boundary = 0.95, scale = TRUE,
                                  global_mean = NULL, global_sd = NULL,
                                  conflict_ratio = 0.25) {
  metric <- match.arg(metric)
  current <- .as_numeric_matrix(current, "current")
  future <- .as_numeric_matrix(future, "future")
  if (!identical(dim(current), dim(future))) {
    stop("current and future must have identical dimensions.", call. = FALSE)
  }
  if (boundary <= 0 || boundary >= 1) {
    stop("boundary must be between 0 and 1.", call. = FALSE)
  }

  occ <- .occupied_index(occupied, nrow(current),
                         threshold = occupied_threshold)
  scaled <- .standardize_pair(current, future, scale, global_mean, global_sd)
  x0 <- scaled$current
  x1 <- scaled$future

  if (is.null(center)) {
    center <- .extract_slot(cnfa, "mf")
  }
  if (is.null(center)) {
    center <- colMeans(x0[occ, , drop = FALSE])
  }
  center <- as.numeric(center)
  if (length(center) != ncol(current)) {
    stop("center must have length equal to ncol(current).", call. = FALSE)
  }

  if (is.null(A)) {
    if (is.null(sensitivity)) {
      sensitivity <- .extract_slot(cnfa, "sf")
    }
    if (is.null(sensitivity)) {
      sensitivity <- rep(1, ncol(current))
    }
    A <- niche_metric(sensitivity = sensitivity, cnfa = cnfa, type = metric)
  }
  A <- .validate_niche_metric(A, ncol(current), "A")

  psi0 <- niche_potential(x0, center = center, A = A)
  psi1 <- niche_potential(x1, center = center, A = A)
  radius0 <- niche_radius(psi0)
  radius1 <- niche_radius(psi1)
  amount <- .climate_change_amount(x0, x1, A = A)
  distance_change <- .niche_distance_change(psi0, psi1)
  composition <- .composition_change(amount, distance_change)
  alignment <- .change_alignment(amount, distance_change)
  b_potential <- as.numeric(stats::quantile(psi0[occ], probs = boundary,
                                            names = FALSE, na.rm = TRUE,
                                            type = 8))
  b_radius <- sqrt(pmax(0, b_potential))
  exceed <- boundary_exceedance(psi1, boundary_value = b_potential,
                                scale = "radial")
  perc <- niche_percentile(psi0, psi1, occupied = occ)
  contrib <- variable_contribution(x0, x1, center = center, A = A)
  mixed <- mixed_variable_response(
    contribution = contrib,
    outside_niche_exceedance = exceed,
    conflict_ratio = conflict_ratio
  )
  class <- classify_exposure(
    climate_change_amount = amount,
    niche_distance_change = distance_change,
    outside_niche_exceedance = exceed,
    composition_change = composition,
    contribution = contrib,
    conflict_ratio = conflict_ratio
  )

  out <- list(
    call = match.call(),
    current = x0,
    future = x1,
    occupied = occ,
    center = center,
    A = A,
    psi_current = psi0,
    psi_future = psi1,
    niche_radius_current = radius0,
    niche_radius_future = radius1,
    climate_change_amount = amount,
    niche_distance_change = distance_change,
    composition_change = composition,
    change_alignment = alignment,
    outside_niche_exceedance = exceed,
    boundary_quantile = boundary,
    boundary_value = b_radius,
    boundary_potential = b_potential,
    boundary_radius = b_radius,
    niche_percentile = perc,
    variable_contribution = contrib,
    mixed_variable_response = mixed,
    classification = class,
    standardization = list(center = scaled$center, scale = scaled$scale)
  )
  class(out) <- "climniche_fit"
  out
}

#' Niche potential
#'
#' @param x Standardized climate matrix.
#' @param center Realized niche center.
#' @param A Niche metric matrix.
#'
#' @return Numeric vector of quadratic niche displacement values.
#' @export
niche_potential <- function(x, center, A) {
  x <- .as_numeric_matrix(x, "x")
  z <- sweep(x, 2L, center, "-")
  .quad_form_rows(z, A)
}

#' Niche radius
#'
#' @param psi Numeric niche-potential values.
#'
#' @return Numeric vector in sensitivity-weighted climate-distance units.
#' @export
niche_radius <- function(psi) {
  sqrt(pmax(0, psi))
}

.climate_change_amount <- function(current, future, A) {
  current <- .as_numeric_matrix(current, "current")
  future <- .as_numeric_matrix(future, "future")
  sqrt(pmax(0, .quad_form_rows(future - current, A)))
}

.composition_change <- function(climate_change_amount, niche_distance_change) {
  sqrt(pmax(0, climate_change_amount^2 - niche_distance_change^2))
}

.change_alignment <- function(climate_change_amount, niche_distance_change) {
  out <- rep(NA_real_, length(climate_change_amount))
  ok <- is.finite(climate_change_amount) &
    climate_change_amount > sqrt(.Machine$double.eps)
  out[ok] <- niche_distance_change[ok] / climate_change_amount[ok]
  out
}

.niche_distance_change <- function(psi_current, psi_future) {
  niche_radius(psi_future) - niche_radius(psi_current)
}

#' Niche boundary exceedance
#'
#' @param psi_future Future niche potential.
#' @param boundary_value Empirical occupied-niche boundary in potential units.
#' @param scale `"radial"` returns exceedance beyond the niche boundary distance;
#'   `"potential"` returns exceedance beyond squared niche potential.
#'
#' @return Numeric vector.
#' @export
boundary_exceedance <- function(psi_future, boundary_value,
                                scale = c("radial", "potential")) {
  scale <- match.arg(scale)
  if (scale == "radial") {
    return(pmax(0, niche_radius(psi_future) - niche_radius(boundary_value)))
  }
  pmax(0, psi_future - boundary_value)
}

#' Niche percentile shift
#'
#' @param psi_current Current niche potential for all cells.
#' @param psi_future Future niche potential for all cells.
#' @param occupied Current occurrence indices used to define the reference CDF.
#'
#' @return Data frame with current, future, and delta percentiles.
#' @export
niche_percentile <- function(psi_current, psi_future, occupied) {
  f <- stats::ecdf(psi_current[occupied])
  current <- f(psi_current)
  future <- f(psi_future)
  data.frame(current = current, future = future, delta = future - current)
}

#' Variable contribution to change in niche potential
#'
#' @param current Current standardized climate matrix.
#' @param future Future standardized climate matrix.
#' @param center Realised niche centre.
#' @param A Niche metric matrix.
#'
#' @return Matrix whose rows sum to the change in niche potential.
#' @export
variable_contribution <- function(current, future, center, A) {
  current <- .as_numeric_matrix(current, "current")
  future <- .as_numeric_matrix(future, "future")
  A <- .validate_niche_metric(A, ncol(current), "A")
  c0 <- sweep(current, 2L, center, "-")
  c1 <- sweep(future, 2L, center, "-")
  out <- c1 * (c1 %*% A) - c0 * (c0 %*% A)
  colnames(out) <- colnames(current)
  out
}

if (requireNamespace("climniche", quietly = TRUE)) {
  library(climniche)
}

sim <- simulate_climniche(n = 500, seed = 10)

toward <- fit_climniche(
  current = sim$current,
  future = sim$future_toward,
  occupied = sim$occupied,
  center = sim$center,
  sensitivity = sim$sensitivity
)

away <- fit_climniche(
  current = sim$current,
  future = sim$future_away,
  occupied = sim$occupied,
  center = sim$center,
  sensitivity = sim$sensitivity
)

away_numeric_occupied <- fit_climniche(
  current = sim$current,
  future = sim$future_away,
  occupied = as.numeric(sim$occupied),
  occupied_threshold = 0.5,
  center = sim$center,
  sensitivity = sim$sensitivity
)

stopifnot(inherits(toward, "climniche_fit"))
stopifnot(identical(away$occupied, away_numeric_occupied$occupied))
stopifnot(length(toward$climate_change_amount) == nrow(sim$current))
stopifnot(length(toward$composition_change) == nrow(sim$current))
stopifnot(length(toward$change_alignment) == nrow(sim$current))
stopifnot(all(toward$composition_change >= -sqrt(.Machine$double.eps)))
stopifnot(all(abs(toward$change_alignment[!is.na(toward$change_alignment)]) <=
                1 + sqrt(.Machine$double.eps)))
stopifnot(isTRUE(all.equal(
  toward$climate_change_amount^2,
  toward$niche_distance_change^2 + toward$composition_change^2,
  tolerance = 1e-7
)))
stopifnot(mean(toward$niche_distance_change) < mean(away$niche_distance_change))
stopifnot(sum(away$outside_niche_exceedance > 0) >=
            sum(toward$outside_niche_exceedance > 0))

A_full <- matrix(c(1, 0.25, 0.25, 1.4), 2, 2)
fit_full <- fit_climniche(
  current = sim$current,
  future = sim$future_away,
  occupied = sim$occupied,
  center = sim$center,
  A = A_full
)
stopifnot(isTRUE(all.equal(rowSums(fit_full$variable_contribution),
                           fit_full$psi_future - fit_full$psi_current,
                           tolerance = 1e-7)))
bad_A <- matrix(c(1, 2, 0, 1), 2, 2)
bad_fit <- try(fit_climniche(
  current = sim$current,
  future = sim$future_away,
  occupied = sim$occupied,
  center = sim$center,
  A = bad_A
), silent = TRUE)
stopifnot(inherits(bad_fit, "try-error"))
stopifnot(is.logical(away$mixed_variable_response))
stopifnot(!("mixed variable response" %in% levels(away$classification)))
diagram <- climniche_diagram_data(away, max_arrows = 50)
stopifnot(inherits(diagram, "climniche_diagram_data"))
stopifnot(nrow(diagram$cells) == length(away$occupied))
stopifnot(nrow(diagram$arrows) <= 50)
stopifnot(all(c("current_axis1", "future_axis1", "class") %in%
                names(diagram$cells)))
showcase <- climniche_showcase_data(away, max_points = 80)
stopifnot(inherits(showcase, "climniche_showcase_data"))
stopifnot(nrow(showcase$plane) <= 80)
stopifnot(all(c("x_mid", "y_mid", "cell_weight") %in%
                names(showcase$plane_bins)))
stopifnot(all(c("metric", "value") %in% names(showcase$metrics)))
stopifnot(all(c("proportion", "count") %in% names(showcase$classes)))
stopifnot(all(c("boundary_quantile", "prop_exceeded") %in%
                names(showcase$boundary)))

if (requireNamespace("raster", quietly = TRUE)) {
  r1 <- raster::raster(nrows = 4, ncols = 4, xmn = 0, xmx = 4,
                       ymn = 0, ymx = 4, crs = "+proj=longlat +datum=WGS84")
  raster::values(r1) <- seq_len(raster::ncell(r1))
  r2 <- r1
  raster::values(r2) <- rev(seq_len(raster::ncell(r2)))
  cur <- raster::stack(r1, r2)
  fut <- raster::stack(r1 + 0.5, r2 + 1)
  occ <- r1
  raster::values(occ) <- c(rep(0, 8), rep(0.7, 4), rep(1, 4))
  domain <- r1
  raster::values(domain) <- c(rep(0, 4), rep(1, 12))

  rf <- fit_climniche_raster(cur, fut, occupied = occ, occupied_threshold = 0.5)
  stopifnot(inherits(rf, "climniche_fit"))
  stopifnot(length(rf$occupied) == 8)
  stopifnot(methods::is(rf$rasters$climate_change_amount, "RasterLayer"))
  rf_domain <- fit_climniche_raster(cur, fut, occupied = occ,
                                    occupied_threshold = 0.5,
                                    domain = domain)
  stopifnot(length(rf_domain$climate_change_amount) == 12)
  stopifnot(length(rf_domain$occupied) == 8)

  if (requireNamespace("ggplot2", quietly = TRUE)) {
    p_map <- plot_climniche_map(rf, occupied = occ, occupied_only = TRUE,
                                occupied_threshold = 0.5)
    p_cls <- plot_climniche_classes(rf, occupied = occ, occupied_only = TRUE,
                                    occupied_threshold = 0.5)
    stopifnot(inherits(p_map, "ggplot"))
    stopifnot(inherits(p_cls, "ggplot"))
    stopifnot(nrow(p_map$data) == 8)
    stopifnot(nrow(p_cls$data) == 8)
    stopifnot(inherits(plot_climniche_variable_contribution(away), "ggplot"))
  }
}

if (requireNamespace("terra", quietly = TRUE)) {
  r1 <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4,
                    ymin = 0, ymax = 4, crs = "EPSG:4326")
  terra::values(r1) <- seq_len(terra::ncell(r1))
  r2 <- r1
  terra::values(r2) <- rev(seq_len(terra::ncell(r2)))
  cur <- c(r1, r2)
  fut <- c(r1 + 0.5, r2 + 1)
  occ <- r1
  terra::values(occ) <- c(rep(0, 8), rep(0.7, 4), rep(1, 4))
  domain <- r1
  terra::values(domain) <- c(rep(0, 4), rep(1, 12))

  tf <- fit_climniche_terra(cur, fut, occupied = occ, occupied_threshold = 0.5)
  stopifnot(inherits(tf, "climniche_fit"))
  stopifnot(length(tf$occupied) == 8)
  stopifnot(methods::is(tf$rasters$climate_change_amount, "SpatRaster"))
  tf_domain <- fit_climniche_terra(cur, fut, occupied = occ,
                                   occupied_threshold = 0.5,
                                   domain = domain)
  stopifnot(length(tf_domain$climate_change_amount) == 12)
  stopifnot(length(tf_domain$occupied) == 8)

  if (requireNamespace("ggplot2", quietly = TRUE)) {
    p_map <- plot_climniche_map(tf, occupied = occ, occupied_only = TRUE,
                                occupied_threshold = 0.5)
    p_cls <- plot_climniche_classes(tf, occupied = occ, occupied_only = TRUE,
                                    occupied_threshold = 0.5)
    stopifnot(inherits(p_map, "ggplot"))
    stopifnot(inherits(p_cls, "ggplot"))
    stopifnot(nrow(p_map$data) == 8)
    stopifnot(nrow(p_cls$data) == 8)
  }
}

---
title: "Examples"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", fig.align = "center")
```

This page uses the Mediterranean anchovy case study from the package data. The
case study is based on a Mediterranean analysis domain, current and projected
sea-surface temperature variables, and an SDM-derived current reference area for
European anchovy (*Engraulis encrasicolus*).

```{r}
library(climniche)

case_path <- system.file("extdata/mediterranean_anchovy", package = "climniche")
summary_tab <- read.csv(file.path(case_path, "anchovy_climniche_summary.csv"))
class_tab <- read.csv(file.path(case_path, "anchovy_climniche_classes.csv"))
variable_tab <- read.csv(file.path(case_path, "anchovy_climniche_top_variables.csv"))
records <- read.csv(file.path(case_path, "anchovy_clean_obis_records.csv"))
```

The included tables keep the vignette small while recording the fitted case.
The full raster workflow used the same package functions shown below.

```{r}
summary_tab[, c(
  "n", "boundary_quantile", "mean_climate_change_amount",
  "mean_niche_distance_change", "prop_niche_divergence",
  "prop_niche_exceedance", "prop_niche_convergence"
)]

head(class_tab)
head(variable_tab)
nrow(records)
```

## Mediterranean maps

The map panel shows the data structure used for interpretation: current
suitability, climate change amount, niche distance change, composition change,
niche boundary exceedance, change alignment, and exposure class.

```{r, echo = FALSE, out.width = "100%"}
knitr::include_graphics("figures/anchovy-mediterranean-maps.png")
```

The map is limited to the Mediterranean analysis domain. The current reference
cells are taken from the SDM suitability layer; exposure is evaluated at those
cells and interpreted against the realised niche estimated from the same
reference set.

## Niche climate diagram

The same result can be shown in climate space. The filled polygon is the current
realised niche envelope, the boundary line is the empirical niche boundary, and
arrows show the mean movement of exposure classes.

```{r, echo = FALSE, out.width = "80%"}
knitr::include_graphics("figures/anchovy-climniche-diagram.png")
```

This diagram is useful when a map alone is ambiguous. It separates cells that
experience large climate change from cells whose future climate is specifically
farther from, closer to, or outside the current realised niche.

## Summary figure

`plot_climniche_showcase()` combines the binned exposure plane, exposure class
proportions, variable contributions, and metric distributions.

```{r, echo = FALSE, out.width = "95%"}
knitr::include_graphics("figures/anchovy-climniche-showcase.png")
```

The Mediterranean case identifies sea-surface temperature minima, maxima, and
mean conditions as the variables contributing most to the increase in niche
distance. The class table gives the same result numerically.

```{r}
class_tab[, c("class", "proportion")]
variable_tab[, c("variable", "mean_contribution", "interpretation")]
```

## Fitting the same data structure

For a raster workflow, current and future environmental layers must have the
same geometry. The occupied layer can be binary or continuous. With a continuous
SDM layer, cells above `occupied_threshold` define the current reference niche.

```r
fit <- fit_climniche_raster(
  current = current_sst_stack,
  future = future_sst_stack,
  occupied = anchovy_sdm_suitability,
  occupied_threshold = 0.40,
  domain = mediterranean_domain,
  sensitivity = c(sst_min = 1, sst_mean = 1, sst_max = 1),
  boundary = 0.95
)

climniche_summary(fit)
plot_climniche_map(fit, metric = "niche_distance_change",
                   occupied = anchovy_sdm_suitability,
                   occupied_only = TRUE,
                   occupied_threshold = 0.40)
plot_climniche_classes(fit, occupied = anchovy_sdm_suitability,
                       occupied_only = TRUE,
                       occupied_threshold = 0.40)
plot_climniche_diagram(fit)
plot_climniche_showcase(fit)
```

For a matrix workflow, extract current and future environmental values first and
pass the current reference cells as a logical vector, row indices, or continuous
suitability vector.

```r
fit <- fit_climniche(
  current = current_values,
  future = future_values,
  occupied = sdm_suitability,
  occupied_threshold = 0.40,
  sensitivity = c(sst_min = 1, sst_mean = 1, sst_max = 1),
  boundary = 0.95
)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{niche_percentile}
\alias{niche_percentile}
\title{Niche percentile shift}
\usage{
niche_percentile(psi_current, psi_future, occupied)
}
\arguments{
\item{psi_current}{Current niche potential for all cells.}

\item{psi_future}{Future niche potential for all cells.}

\item{occupied}{Current occurrence indices used to define the reference CDF.}
}
\value{
Data frame with current, future, and delta percentiles.
}
\description{
Niche percentile shift
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{niche_potential}
\alias{niche_potential}
\title{Niche potential}
\usage{
niche_potential(x, center, A)
}
\arguments{
\item{x}{Standardized climate matrix.}

\item{center}{Realized niche center.}

\item{A}{Niche metric matrix.}
}
\value{
Numeric vector of quadratic niche displacement values.
}
\description{
Niche potential
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualise.R
\name{plot_climniche_variable_contribution}
\alias{plot_climniche_variable_contribution}
\title{Plot mean variable contribution}
\usage{
plot_climniche_variable_contribution(
  x,
  occupied_only = TRUE,
  variable_labels = NULL,
  title = NULL
)
}
\arguments{
\item{x}{A \code{climniche_fit} object.}

\item{occupied_only}{If TRUE, summarize occupied cells only.}

\item{variable_labels}{Optional named vector replacing variable labels.}

\item{title}{Optional plot title. Use \code{FALSE} to suppress it.}
}
\value{
A ggplot object.
}
\description{
Plot mean variable contribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagram.R
\name{climniche_diagram_data}
\alias{climniche_diagram_data}
\title{Build data for a niche climate exposure diagram}
\usage{
climniche_diagram_data(
  x,
  scope = c("current", "all"),
  max_arrows = 350L,
  seed = 1L
)
}
\arguments{
\item{x}{A fitted climniche object.}

\item{scope}{\code{"current"} for current occurrence, range or thresholded SDM
cells; \code{"all"} for all evaluated cells.}

\item{max_arrows}{Maximum number of current to future arrows to keep.}

\item{seed}{Random seed used when subsampling arrows.}
}
\value{
A list of data frames used by \code{plot_climniche_diagram()}.
}
\description{
Build data for a niche climate exposure diagram
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{fit_climniche}
\alias{fit_climniche}
\title{Fit niche climate exposure}
\usage{
fit_climniche(
  current,
  future,
  occupied = NULL,
  occupied_threshold = 0,
  cnfa = NULL,
  center = NULL,
  sensitivity = NULL,
  A = NULL,
  metric = c("diag", "factor"),
  boundary = 0.95,
  scale = TRUE,
  global_mean = NULL,
  global_sd = NULL,
  conflict_ratio = 0.25
)
}
\arguments{
\item{current}{Numeric matrix or data frame of current environmental values.}

\item{future}{Numeric matrix or data frame of future environmental values.}

\item{occupied}{NULL, logical vector, row indices, or a numeric vector with
one value per row identifying current occurrence, range, or thresholded SDM
cells.}

\item{occupied_threshold}{Threshold used when \code{occupied} contains binary or
continuous values.}

\item{cnfa}{Optional CENFA model object.}

\item{center}{Optional realised niche centre in standardised climate space.}

\item{sensitivity}{Optional environmental sensitivity weights.}

\item{A}{Optional niche metric matrix.}

\item{metric}{Metric construction when \code{A} is not supplied.}

\item{boundary}{Quantile defining the empirical realised niche boundary.}

\item{scale}{Logical. If TRUE, standardise current and future values.}

\item{global_mean}{Optional means used for standardisation.}

\item{global_sd}{Optional standard deviations used for standardisation.}

\item{conflict_ratio}{Minimum minority sign contribution share used to mark
mixed variable responses. Set to NULL to disable this flag.}
}
\value{
An object of class \code{climniche_fit}.
}
\description{
Fit niche climate exposure
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{plot_climniche_distribution}
\alias{plot_climniche_distribution}
\title{Plot a climniche metric distribution}
\usage{
plot_climniche_distribution(
  x,
  metric = c("niche_distance_change", "climate_change_amount",
    "outside_niche_exceedance", "composition_change"),
  scope = c("current", "all"),
  title = NULL
)
}
\arguments{
\item{x}{A fitted climniche object.}

\item{metric}{Metric to plot.}

\item{scope}{\code{"current"} for current occurrence/range cells or \code{"all"} for
all evaluated cells.}

\item{title}{Optional plot title.}
}
\value{
A ggplot object.
}
\description{
Plot a climniche metric distribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{plot_climniche_classes}
\alias{plot_climniche_classes}
\title{Plot climniche classes}
\usage{
plot_climniche_classes(
  x,
  occupied = NULL,
  occupied_only = FALSE,
  occupied_threshold = 0,
  title = NULL
)
}
\arguments{
\item{x}{A fitted climniche object with raster outputs.}

\item{occupied}{Optional current occurrence/range RasterLayer to overlay.}

\item{occupied_only}{If TRUE, mask the plotted classes to current occurrence
cells.}

\item{occupied_threshold}{Threshold used when \code{occupied} contains binary or
continuous values.}

\item{title}{Optional plot title. Use \code{FALSE} to suppress it.}
}
\value{
A ggplot object.
}
\description{
Plot climniche classes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{climniche_report}
\alias{climniche_report}
\title{Build a climniche report}
\usage{
climniche_report(
  x,
  species = NULL,
  scope = c("current", "all"),
  top_variables = 5
)
}
\arguments{
\item{x}{A fitted \code{climniche_fit} object.}

\item{species}{Optional species name used in printed reports.}

\item{scope}{\code{"current"} for current occurrence/range cells or \code{"all"} for
all evaluated cells.}

\item{top_variables}{Number of variable contributions to show.}
}
\value{
An object of class \code{climniche_report}.
}
\description{
Build a climniche report
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{variable_contribution}
\alias{variable_contribution}
\title{Variable contribution to change in niche potential}
\usage{
variable_contribution(current, future, center, A)
}
\arguments{
\item{current}{Current standardized climate matrix.}

\item{future}{Future standardized climate matrix.}

\item{center}{Realised niche centre.}

\item{A}{Niche metric matrix.}
}
\value{
Matrix whose rows sum to the change in niche potential.
}
\description{
Variable contribution to change in niche potential
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{plot_climniche_class_summary}
\alias{plot_climniche_class_summary}
\title{Plot climniche class proportions}
\usage{
plot_climniche_class_summary(x, scope = c("current", "all"), title = NULL)
}
\arguments{
\item{x}{A fitted climniche object.}

\item{scope}{\code{"current"} for current occurrence/range cells or \code{"all"} for
all evaluated cells.}

\item{title}{Optional plot title.}
}
\value{
A ggplot object.
}
\description{
Plot climniche class proportions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metric.R
\name{niche_metric}
\alias{niche_metric}
\title{Build a sensitivity-weighted niche metric}
\usage{
niche_metric(sensitivity = NULL, cnfa = NULL, type = c("diag", "factor"))
}
\arguments{
\item{sensitivity}{Numeric vector of climate-variable sensitivity weights.}

\item{cnfa}{Optional CENFA \code{cnfa} object or list with \code{sf}, \code{co}, and \code{eig}.}

\item{type}{Metric type. \code{"diag"} uses variable-level sensitivity weights.
\code{"factor"} uses a factor metric when CNFA factor coordinates are available.}
}
\value{
A positive semi-definite matrix.
}
\description{
Build a sensitivity-weighted niche metric
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{plot_climniche_map}
\alias{plot_climniche_map}
\title{Plot a climniche map}
\usage{
plot_climniche_map(
  x,
  metric = c("niche_distance_change", "outside_niche_exceedance",
    "climate_change_amount", "composition_change", "change_alignment"),
  occupied = NULL,
  occupied_only = FALSE,
  occupied_threshold = 0,
  title = NULL,
  midpoint = 0
)
}
\arguments{
\item{x}{A fitted climniche object with raster outputs, or a RasterLayer.}

\item{metric}{Metric to plot.}

\item{occupied}{Optional current occurrence/range RasterLayer to overlay.}

\item{occupied_only}{If TRUE, mask the plotted raster to current occurrence
cells.}

\item{occupied_threshold}{Threshold used when \code{occupied} contains binary or
continuous values.}

\item{title}{Optional plot title. Use \code{FALSE} to suppress it.}

\item{midpoint}{Midpoint for the niche distance change colour scale.}
}
\value{
A ggplot object.
}
\description{
Plot a climniche map
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{write_climniche_report}
\alias{write_climniche_report}
\title{Write a climniche report to Markdown}
\usage{
write_climniche_report(report, file)
}
\arguments{
\item{report}{An object returned by \code{\link[=climniche_report]{climniche_report()}}.}

\item{file}{Output Markdown file.}
}
\value{
Invisibly returns \code{file}.
}
\description{
Write a climniche report to Markdown
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{plot_climniche_exposure}
\alias{plot_climniche_exposure}
\title{Plot the climniche exposure plane}
\usage{
plot_climniche_exposure(
  x,
  scope = c("current", "all"),
  max_points = 6000,
  seed = 1,
  title = NULL
)
}
\arguments{
\item{x}{A fitted climniche object.}

\item{scope}{\code{"current"} for current occurrence/range cells or \code{"all"} for
all evaluated cells.}

\item{max_points}{Maximum number of points to draw.}

\item{seed}{Random seed used when subsampling.}

\item{title}{Optional plot title.}
}
\value{
A ggplot object.
}
\description{
Plot the climniche exposure plane
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{plot_climniche_report}
\alias{plot_climniche_report}
\title{Plot a climniche report figure}
\usage{
plot_climniche_report(x, scope = c("current", "all"))
}
\arguments{
\item{x}{A fitted climniche object.}

\item{scope}{\code{"current"} for current occurrence/range cells or \code{"all"} for
all evaluated cells.}
}
\value{
A patchwork object when \code{patchwork} is installed, otherwise a named
list of ggplot objects.
}
\description{
Plot a climniche report figure
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{simulate_climniche}
\alias{simulate_climniche}
\title{Simulate a minimal climate niche change experiment}
\usage{
simulate_climniche(
  n = 2000,
  p = 2,
  seed = 1,
  rho = 0,
  prevalence = 0.3,
  shift = 0.4
)
}
\arguments{
\item{n}{Number of climate cells.}

\item{p}{Number of climate variables.}

\item{seed}{Random seed.}

\item{rho}{Pairwise correlation among simulated climate variables.}

\item{prevalence}{Proportion of background cells treated as true current
occurrence locations under the virtual niche.}

\item{shift}{Climate change amount imposed in the closer to niche and
farther from niche scenarios.}
}
\value{
A list with current, future_toward, future_away, occupied, center,
sensitivity and A.
}
\description{
Simulate a minimal climate niche change experiment
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{niche_radius}
\alias{niche_radius}
\title{Niche radius}
\usage{
niche_radius(psi)
}
\arguments{
\item{psi}{Numeric niche-potential values.}
}
\value{
Numeric vector in sensitivity-weighted climate-distance units.
}
\description{
Niche radius
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/showcase.R
\name{climniche_showcase_data}
\alias{climniche_showcase_data}
\title{Build data for the climniche showcase figure}
\usage{
climniche_showcase_data(
  x,
  scope = c("current", "all"),
  max_points = 6000L,
  seed = 1L,
  plane_bins = 45L,
  boundary_probs = seq(0.5, 0.99, 0.01),
  top_variables = 6L
)
}
\arguments{
\item{x}{A fitted climniche object.}

\item{scope}{\code{"current"} for current occurrence, range or thresholded SDM
cells; \code{"all"} for all evaluated cells.}

\item{max_points}{Maximum number of cells to keep for the exposure plane.}

\item{seed}{Random seed used when subsampling cells.}

\item{plane_bins}{Number of fixed bins used to summarize the exposure plane.}

\item{boundary_probs}{Boundary quantiles used for the sensitivity curve.}

\item{top_variables}{Number of variables to show.}
}
\value{
A list of data frames used by \code{plot_climniche_showcase()}.
}
\description{
Build data for the climniche showcase figure
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagram.R
\name{plot_climniche_diagram}
\alias{plot_climniche_diagram}
\title{Plot a niche climate exposure diagram}
\usage{
plot_climniche_diagram(
  x,
  scope = c("current", "all"),
  type = c("summary", "sample"),
  max_arrows = 350L,
  seed = 1L,
  show_reference = FALSE,
  show_hulls = TRUE,
  boundary_shape = c("hull", "circle", "none"),
  show_boundary_label = TRUE,
  show_points = NULL,
  show_endpoints = FALSE,
  show_center = FALSE,
  show_variables = FALSE,
  variable_labels = NULL,
  title = NULL
)
}
\arguments{
\item{x}{A fitted climniche object or data returned by
\code{climniche_diagram_data()}.}

\item{scope}{\code{"current"} for current occurrence, range or thresholded SDM
cells; \code{"all"} for all evaluated cells.}

\item{type}{\code{"summary"} draws class mean arrows; \code{"sample"} draws sampled
cell arrows and future points.}

\item{max_arrows}{Maximum number of current to future arrows to draw when
\code{type = "sample"}.}

\item{seed}{Random seed used when subsampling arrows.}

\item{show_reference}{Logical; draw the full analysed environmental domain.}

\item{show_hulls}{Logical; draw current and future niche hulls.}

\item{boundary_shape}{Boundary display. \code{"hull"} draws the empirical
occupied niche polygon, \code{"circle"} draws a constant niche distance boundary,
and \code{"none"} suppresses the boundary.}

\item{show_boundary_label}{Logical; add reference-area explanations below
the exposure-class legend.}

\item{show_points}{Logical; draw future points when \code{type = "sample"}.}

\item{show_endpoints}{Logical; draw class mean future positions when
\code{type = "summary"}.}

\item{show_center}{Logical; mark the realised niche centre.}

\item{show_variables}{Logical; draw environmental variable directions.}

\item{variable_labels}{Optional named vector replacing variable labels.}

\item{title}{Optional plot title.}
}
\value{
A ggplot object.
}
\description{
Plot a niche climate exposure diagram
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{climniche_summary}
\alias{climniche_summary}
\title{Summarise climniche results}
\usage{
climniche_summary(x, scope = c("current", "all"))
}
\arguments{
\item{x}{A fitted \code{climniche_fit} object.}

\item{scope}{\code{"current"} for current occurrence/range cells or \code{"all"} for
all evaluated cells.}
}
\value{
A one-row data frame with key report metrics.
}
\description{
Summarise climniche results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{fit_climniche_terra}
\alias{fit_climniche_terra}
\title{Fit climniche to terra raster data}
\usage{
fit_climniche_terra(
  current,
  future,
  occupied = NULL,
  occupied_threshold = 0,
  domain = NULL,
  domain_threshold = 0,
  ...
)
}
\arguments{
\item{current}{terra SpatRaster of current environmental layers.}

\item{future}{terra SpatRaster of future environmental layers.}

\item{occupied}{Optional one layer SpatRaster with binary or continuous
occurrence, range, or SDM suitability values.}

\item{occupied_threshold}{Values greater than this threshold are treated as
current occurrence cells.}

\item{domain}{Optional one layer SpatRaster limiting cells where exposure is
analysed.}

\item{domain_threshold}{Values greater than this threshold define the domain.}

\item{...}{Additional arguments passed to \code{fit_climniche()}.}
}
\value{
An object of class \code{climniche_fit} with raster outputs.
}
\description{
Fit climniche to terra raster data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{boundary_exceedance}
\alias{boundary_exceedance}
\title{Niche boundary exceedance}
\usage{
boundary_exceedance(
  psi_future,
  boundary_value,
  scale = c("radial", "potential")
)
}
\arguments{
\item{psi_future}{Future niche potential.}

\item{boundary_value}{Empirical occupied-niche boundary in potential units.}

\item{scale}{\code{"radial"} returns exceedance beyond the niche boundary distance;
\code{"potential"} returns exceedance beyond squared niche potential.}
}
\value{
Numeric vector.
}
\description{
Niche boundary exceedance
}
